@@ -1,4 +1,4 @@
-//@line 41 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/base/content/contentAreaClick.js"
+//@line 41 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/base/content/contentAreaClick.js"
 
 /*
  * - [ Dependencies ] ---------------------------------------------------------
@@ -7,32 +7,44 @@
  */
 
   var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
-
-  function linkNodeForClickEvent(event)
+  
+  /** 
+   * extract the href from the link click event. 
+   * We look for HTMLAnchorElement, HTMLAreaElement, HTMLLinkElement,
+   * HTMLInputElement.form.action, and nested anchor tags.
+   * 
+   * @return href for the url being clicked
+   */
+  function hRefForClickEvent(event)
   {
     var target = event.target;
-    var linkNode;
-    var linkNodeText;
-
+    var href;
     var isKeyPress = (event.type == "keypress");
 
-    if ( target instanceof HTMLAnchorElement ||
-         target instanceof HTMLAreaElement   ||
-         target instanceof HTMLLinkElement ) {
+    if (target instanceof HTMLAnchorElement ||
+        target instanceof HTMLAreaElement   ||
+        target instanceof HTMLLinkElement)
+    {
       if (target.hasAttribute("href")) 
-        linkNode = target;
+        href = target.href;
     }
-    else if (!(target instanceof HTMLInputElement)) {
+    else if (target instanceof HTMLInputElement)
+    {
+      if (target.form && target.form.action)
+        href = target.form.action;      
+    }
+    else 
+    {
+      // we may be nested inside of a link node
       linkNode = event.originalTarget;
       while (linkNode && !(linkNode instanceof HTMLAnchorElement))
         linkNode = linkNode.parentNode;
-      // <a> cannot be nested.  So if we find an anchor without an
-      // href, there is no useful <a> around the target
-      if (linkNode && !linkNode.hasAttribute("href"))
-        linkNode = null;
+      
+      if (linkNode)
+        href = linkNode.href;
     }
 
-    return linkNode;
+    return href;
   }
 
   // Called whenever the user clicks in the content area,
@@ -40,17 +52,14 @@
   // should always return true for click to go through
   function contentAreaClick(event) 
   {
-    var linkNode = linkNodeForClickEvent(event);
-    if (linkNode && linkNode.href) 
+    var href = hRefForClickEvent(event);
+    if (href) 
     {
-      handleLinkClick(event, linkNode.href, null);
-
-      // block the link click if we determine that this URL
-      // is phishy (i.e. a potential email scam) 
+      handleLinkClick(event, href, null);
       if (!event.button)  // left click only
-        return !isPhishingURL(linkNode, false); 
+        return gPhishingDetector.warnOnSuspiciousLinkClick(href); // let the phishing detector check the link
     }
-
+    
     return true;
   }
 
@@ -61,10 +70,18 @@
     return false;
   }
 
+  function getContentFrameURI(aFocusedWindow)
+  {
+    var contentFrame = isContentFrame(aFocusedWindow) ? aFocusedWindow : window.content;
+    return contentFrame.location.href;
+  }
+
   function handleLinkClick(event, href, linkNode)
   {
     // Make sure we are allowed to open this URL
-    urlSecurityCheck(href, document);
+    var focusedWindow = document.commandDispatcher.focusedWindow;
+    var sourceURL = getContentFrameURI(focusedWindow);
+    urlSecurityCheck(href, sourceURL);
     return false;
   }
 
