@@ -1,4 +1,4 @@
-//@line 37 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/extensions/newsblog/content/Feed.js"
+//@line 37 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/extensions/newsblog/content/Feed.js"
 
 
 // error codes used to inform the consumer about attempts to download a feed
@@ -213,8 +213,11 @@ Feed.prototype =
     return title;
   },
 
-  set title (aNewTitle) 
+  set title (aNewTitle)
   {
+    if (!aNewTitle)
+      return;
+
     var ds = getSubscriptionsDS(this.server);
     aNewTitle = rdf.GetLiteral(aNewTitle);
     var old_title = ds.GetTarget(this.resource, DC_TITLE, true);
@@ -283,6 +286,9 @@ Feed.prototype =
 
   set link (aNewLink)
   {
+    if (!aNewLink)
+      return;
+
     var ds = getSubscriptionsDS(this.server);
     aNewLink = rdf.GetLiteral(aNewLink);
     var old_link = ds.GetTarget(this.resource, RSS_LINK, true);
