@@ -1,4 +1,4 @@
-//@line 37 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/extensions/newsblog/content/FeedItem.js"
+//@line 37 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/extensions/newsblog/content/FeedItem.js"
 
 // Handy conversion values.
 const HOURS_TO_MINUTES = 60;
@@ -40,9 +40,9 @@ const REMOTE_STYLE = "\n\
       iframe {\n\
         position: fixed;\n\
         top: 0;\n\
-        right: 0;\n\
-        bottom: 0;\n\
         left: 0;\n\
+        width: 100%;\n\
+        height: 100%;\n\
         border: none;\n\
       }\n\
 ";
@@ -217,6 +217,16 @@ FeedItem.prototype =
     var itemResource = rdf.GetResource(itemURI);
 
     var downloaded = ds.GetTarget(itemResource, FZ_STORED, true);
+
+    // Backward compatibility: we might have stored this item before isStoredWithId
+    // has been turned on for RSS 2.0 (bug 354345). Check whether this item has been
+    // stored with its URL.
+    if (!downloaded && itemURI != this.mURL)
+    {
+      itemResource = rdf.GetResource(this.mURL);
+      downloaded = ds.GetTarget(itemResource, FZ_STORED, true);
+    }
+
     if (!downloaded || downloaded.QueryInterface(Components.interfaces.nsIRDFLiteral).Value == "false") 
     {
       // HACK ALERT: before we give up, try to work around an entity escaping bug in RDF
@@ -254,6 +264,12 @@ FeedItem.prototype =
     var itemURI = this.itemUniqueURI;
     var resource = rdf.GetResource(itemURI);
     
+    // Backward compatibility: we might have stored this item before isStoredWithId
+    // has been turned on for RSS 2.0 (bug 354345). Check whether this item has been
+    // stored with its URL.
+    if (!ds.GetTarget(resource, FZ_STORED, true) && itemURI != this.mURL)
+      resource = rdf.GetResource(this.mURL);
+
     if (!ds.HasAssertion(resource, FZ_FEED, rdf.GetResource(this.feed.url), true))
       ds.Assert(resource, FZ_FEED, rdf.GetResource(this.feed.url), true);
     
@@ -354,6 +370,7 @@ FeedItem.prototype =
       openingLine +
       'X-Mozilla-Status: 0000\n' +
       'X-Mozilla-Status2: 00000000\n' +
+      'X-Mozilla-Keys:                                                                                \n' +
       'Date: ' + this.mDate + '\n' +
       'Message-Id: <' + this.messageID + '>\n' +
       'From: ' + this.author + '\n' +
@@ -383,9 +400,11 @@ FeedItem.prototype =
 
     // Get the folder and database storing the feed's messages and headers.
     folder = this.feed.folder.QueryInterface(Components.interfaces.nsIMsgLocalMailFolder);
-
+    var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
+    msgFolder.gettingNewMessages = true;
     // source is a unicode string, we want to save a char * string in the original charset. So convert back
     folder.addMessage(this.mUnicodeConverter.ConvertFromUnicode(source));
+    msgFolder.gettingNewMessages = false;
     this.markStored();
   }
 };
@@ -432,86 +451,4 @@ FeedEnclosure.prototype =
   }
 };
 
-function W3CToIETFDate(dateString) {
-  // Converts a W3C-DTF (subset of ISO 8601) date string to an IETF date string.
-  // W3C-DTF is described in this note: http://www.w3.org/TR/NOTE-datetime
-  // IETF is obtained via the Date object's toUTCString() method.  The object's
-  // toString() method is insufficient because it spells out timezones on Win32
-  // (f.e. "Pacific Standard Time" instead of "PST"), which Mail doesn't grok.
-  // For info, see http://lxr.mozilla.org/mozilla/source/js/src/jsdate.c#1526.
 
-  var parts = dateString.match(/(\d\d\d\d)(-(\d\d))?(-(\d\d))?(T(\d\d):(\d\d)(:(\d\d)(\.(\d+))?)?(Z|([+-])(\d\d):(\d\d))?)?/);
-  debug("date parts: " + parts);
-
-  // Here's an example of a W3C-DTF date string and what .match returns for it.
-  //  date: 2003-05-30T11:18:50.345-08:00
-  //  date.match returns array values:
-  //   0: 2003-05-30T11:18:50-08:00,
-  //   1: 2003,
-  //   2: -05,
-  //   3: 05,
-  //   4: -30,
-  //   5: 30,
-  //   6: T11:18:50-08:00,
-  //   7: 11,
-  //   8: 18,
-  //   9: :50,
-  //   10: 50,
-  //   11: .345,
-  //   12: 345,
-  //   13: -08:00,
-  //   14: -,
-  //   15: 08,
-  //   16: 00
-
-  // Create a Date object from the date parts.  Note that the Date object
-  // apparently can't deal with empty string parameters in lieu of numbers,
-  // so optional values (like hours, minutes, seconds, and milliseconds)
-  // must be forced to be numbers.
-  var date = new Date(parts[1], parts[3]-1, parts[5], parts[7] || 0,
-                      parts[8] || 0, parts[10] || 0, parts[12] || 0);
-
-  // We now have a value that the Date object thinks is in the local timezone
-  // but which actually represents the date/time in the remote timezone
-  // (f.e. the value was "10:00 EST", and we have converted it to "10:00 PST"
-  // instead of "07:00 PST").  We need to correct that.  To do so, we're going
-  // to add the offset between the remote timezone and UTC (to convert the value
-  // to UTC), then add the offset between UTC and the local timezone (to convert
-  // the value to the local timezone).
-
-  // Ironically, W3C-DTF gives us the offset between UTC and the remote timezone
-  // rather than the other way around, while the getTimezoneOffset() method
-  // of a Date object gives us the offset between the local timezone and UTC
-  // rather than the other way around.  Both of these are the additive inverse
-  // (i.e. -x for x) of what we want, so we have to invert them to use them
-  // by multipying by -1
-  // (f.e. if "the offset between UTC and the remote timezone" is -5 hours,
-  // then "the offset between the remote timezone and UTC" is -5*-1 = 5 hours).
-
-  // Note that if the timezone portion of the date/time string is absent
-  // (which violates W3C-DTF, although ISO 8601 allows it), we assume the value
-  // to be in UTC.
-
-  // The offset between the remote timezone and UTC in milliseconds.
-  var remoteToUTCOffset = 0;
-  if (parts[13] && parts[13] != "Z") {
-    var direction = (parts[14] == "+" ? 1 : -1);
-    if (parts[15])
-      remoteToUTCOffset += direction * parts[15] * HOURS_TO_MILLISECONDS;
-    if (parts[16])
-      remoteToUTCOffset += direction * parts[16] * MINUTES_TO_MILLISECONDS;
-  }
-  remoteToUTCOffset = remoteToUTCOffset * -1; // invert it
-  debug("date remote to UTC offset: " + remoteToUTCOffset);
-
-  // The offset between UTC and the local timezone in milliseconds.
-  var UTCToLocalOffset = date.getTimezoneOffset() * MINUTES_TO_MILLISECONDS;
-  UTCToLocalOffset = UTCToLocalOffset * -1; // invert it
-  debug("date UTC to local offset: " + UTCToLocalOffset);
-
-  date.setTime(date.getTime() + remoteToUTCOffset + UTCToLocalOffset);
-
-  debug("date string: " + date.toUTCString());
-
-  return date.toUTCString();
-}
