@@ -1,4 +1,4 @@
-//@line 38 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/extensions/newsblog/content/feed-subscriptions.js"
+//@line 38 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/extensions/newsblog/content/feed-subscriptions.js"
 
 const MSG_FOLDER_FLAG_TRASH = 0x0100;
 const IPS = Components.interfaces.nsIPromptService;
@@ -819,7 +819,7 @@ var gFeedSubscriptionsWindow = {
       else if(rv)
       {
         //debug("opml:\n"+serial.serializeToString(opmlDoc)+"\n");
-        var file = new LocalFile(rv.file, MODE_WRONLY | MODE_CREATE);
+        var file = new LocalFile(rv.file, MODE_WRONLY | MODE_CREATE | MODE_TRUNCATE);
         serial.serializeToStream(opmlDoc,file.outputStream,'utf-8');
         file.close();
       }
