@@ -1,4 +1,4 @@
-//@line 36 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/base/content/ABSearchDialog.js"
+//@line 36 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/base/content/ABSearchDialog.js"
 
 var searchSessionContractID = "@mozilla.org/messenger/searchSession;1";
 var gSearchSession;
@@ -45,7 +45,7 @@ function searchOnLoad()
 
   gSearchBundle = document.getElementById("bundle_search");
   gSearchStopButton.setAttribute("label", gSearchBundle.getString("labelForSearchButton"));
-  gSearchStopButton.setAttribute("accesskey", gSearchBundle.getString("accesskeyForSearchButton"));
+  gSearchStopButton.setAttribute("accesskey", gSearchBundle.getString("labelForSearchButton.accesskey"));
   gAddressBookBundle = document.getElementById("bundle_addressBook");
   gSearchSession = Components.classes[searchSessionContractID].createInstance(Components.interfaces.nsIMsgSearchSession);
 
@@ -78,6 +78,8 @@ function initializeSearchWindowWidgets()
   gPropertiesButton = document.getElementById("propertiesButton");
   gComposeButton = document.getElementById("composeButton");
   gStatusText = document.getElementById('statusText');
+  // matchAll doesn't make sense for address book search
+  hideMatchAllItem();
 }
 
 function onSearchStop() 
