@@ -1,9 +1,10 @@
-//@line 41 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/components/addrbook/content/abCommon.js"
+//@line 41 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/components/addrbook/content/abCommon.js"
 
 var dirTree = 0;
 var abList = 0;
 var gAbResultsTree = null;
 var gAbView = null;
+var gAddressBookBundle;
 
 var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
 var gPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
@@ -225,8 +226,7 @@ function AbEditSelectedDirectory()
       goEditListDialog(dirUri, null, selecteduri, UpdateCardView);
     }
     else {
-      var properties = directory.directoryProperties;
-      if (properties.dirType == kLDAPDirectory) {
+      if (directory instanceof Components.interfaces.nsIAbLDAPDirectory) {
         var ldapUrlPrefix = "moz-abldapdirectory://";
         var args = { selectedDirectory: directory.dirName,
                      selectedDirectoryString: null};
@@ -260,6 +260,7 @@ function InitCommonJS()
   dirTree = document.getElementById("dirTree");
   abList = document.getElementById("addressbookList");
   gAbResultsTree = document.getElementById("abResultsTree");
+  gAddressBookBundle = document.getElementById("bundle_addressBook");
 }
 
 function SetupAbCommandUpdateHandlers()
@@ -597,7 +598,7 @@ function DirPaneClick(event)
 
   // if the user clicks on the header / trecol, do nothing
   if (event.originalTarget.localName == "treecol") {
-    event.preventBubble();
+    event.stopPropagation();
     return;
   }
 }
@@ -1002,9 +1003,8 @@ var gQuickSearchFocusEl = null;
 
 function onClearSearch()
 {
-  if (!gSearchInput.showingSearchCriteria) // ignore the text box value if it's just showing the search criteria string
+  if (gSearchInput && !gSearchInput.showingSearchCriteria) // ignore the text box value if it's just showing the search criteria string
   {
-//     gQuickSearchFocusEl = gLastFocusedElement;  //save of the last focused element so that focus can be restored
      onAbClearSearch();
      // this needs to be on a timer otherwise we end up messing up the focus while the Search("") is still happening
      setTimeout("restoreSearchFocusAfterClear();", 0); 
