@@ -1,4 +1,4 @@
-//@line 40 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/base/content/mail3PaneWindowCommands.js"
+//@line 40 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/base/content/mail3PaneWindowCommands.js"
 
 var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
 var gMessengerBundle = document.getElementById("bundle_messenger");
@@ -17,7 +17,7 @@ var FolderPaneController =
 			case "cmd_copy":
 			case "cmd_paste":
 				return true;
-				
+
 			default:
 				return false;
 		}
@@ -25,7 +25,7 @@ var FolderPaneController =
 
 	isCommandEnabled: function(command)
 	{
-    if (IsFakeAccount()) 
+    if (IsFakeAccount())
       return false;
 
 		switch ( command )
@@ -61,7 +61,7 @@ var FolderPaneController =
 				}
 				catch (ex) {
 					//dump("specialFolder failure: " + ex + "\n");
-				} 
+				}
         if (specialFolder == "Inbox" || specialFolder == "Trash" || specialFolder == "Drafts" ||
             specialFolder == "Sent" || specialFolder == "Templates" || specialFolder == "Unsent Messages" ||
             (specialFolder == "Junk" && !CanRenameDeleteJunkMail(GetSelectedFolderURI())) || isServer == "true")
@@ -92,7 +92,7 @@ var FolderPaneController =
 				break;
 		}
 	},
-	
+
 	onEvent: function(event)
 	{
 	}
@@ -133,6 +133,10 @@ var DefaultController =
 			case "cmd_previousMsg":
 			case "cmd_previousUnreadMsg":
 			case "cmd_previousFlaggedMsg":
+      case "button_goForward":
+      case "button_goBack":
+      case "cmd_goForward":
+      case "cmd_goBack":
       case "cmd_goStartPage":
 			case "cmd_viewAllMsgs":
 			case "cmd_viewUnreadMsgs":
@@ -162,9 +166,10 @@ var DefaultController =
 			case "cmd_getNextNMessages":
 			case "cmd_find":
 			case "cmd_findAgain":
-			case "cmd_findPrev":
+			case "cmd_findPrevious":
       case "cmd_search":
       case "button_mark":
+      case "cmd_tag":
 			case "cmd_markAsRead":
 			case "cmd_markAllRead":
 			case "cmd_markThreadAsRead":
@@ -176,12 +181,6 @@ var DefaultController =
       case "cmd_applyFilters":
       case "cmd_runJunkControls":
       case "cmd_deleteJunk":
-      case "cmd_label0":
-      case "cmd_label1":
-      case "cmd_label2":
-      case "cmd_label3":
-      case "cmd_label4":
-      case "cmd_label5":
       case "button_file":
 			case "cmd_file":
 			case "cmd_emptyTrash":
@@ -190,11 +189,12 @@ var DefaultController =
       case "cmd_close":
       case "cmd_selectAll":
       case "cmd_selectThread":
+      case "cmd_moveToFolderAgain":
 				return true;
       case "cmd_downloadFlagged":
       case "cmd_downloadSelected":
       case "cmd_synchronizeOffline":
-        return(CheckOnline());
+        return MailOfflineMgr.isOnline();
 
       case "cmd_watchThread":
       case "cmd_killThread":
@@ -211,7 +211,7 @@ var DefaultController =
     enabled.value = false;
     var checkStatus = new Object();
 
-    if (IsFakeAccount()) 
+    if (IsFakeAccount())
       return false;
 
     switch ( command )
@@ -303,14 +303,9 @@ var DefaultController =
           gDBView.getCommandStatus(nsMsgViewCommandType.deleteJunk, enabled, checkStatus);
         return enabled.value;
       case "button_mark":
+      case "cmd_tag":
       case "cmd_markAsRead":
       case "cmd_markThreadAsRead":
-      case "cmd_label0":
-      case "cmd_label1":
-      case "cmd_label2":
-      case "cmd_label3":
-      case "cmd_label4":
-      case "cmd_label5":
         return GetNumSelectedMessages() > 0;
       case "button_previous":
       case "button_next":
@@ -321,6 +316,13 @@ var DefaultController =
       case "cmd_previousMsg":
       case "cmd_previousUnreadMsg":
         return IsViewNavigationItemEnabled();
+      case "button_goForward":
+      case "button_goBack":
+      case "cmd_goForward":
+      case "cmd_goBack":
+        if (gDBView)
+          enabled.value = gDBView.navigateStatus((command == "cmd_goBack" || command == "button_goBack") ? nsMsgNavigationType.back : nsMsgNavigationType.forward);
+        return enabled.value;
       case "cmd_goStartPage":
         return pref.getBoolPref("mailnews.start_page.enabled") && !IsMessagePaneCollapsed();
       case "cmd_markAllRead":
@@ -328,7 +330,7 @@ var DefaultController =
         return IsFolderSelected();
       case "cmd_find":
       case "cmd_findAgain":
-      case "cmd_findPrev":
+      case "cmd_findPrevious":
         return IsMessageDisplayedInMessagePane();
         break;
       case "cmd_search":
@@ -340,7 +342,7 @@ var DefaultController =
         if (GetNumSelectedMessages() <= 0) return false;
       case "cmd_expandAllThreads":
       case "cmd_collapseAllThreads":
-        if (!gDBView || !gDBView.supportsThreading) 
+        if (!gDBView || !gDBView.supportsThreading)
           return false;
         return (gDBView.viewFlags & nsMsgViewFlagsType.kThreadedDisplay);
       case "cmd_nextFlaggedMsg":
@@ -378,13 +380,15 @@ var DefaultController =
       case "cmd_close":
         return true;
       case "cmd_downloadFlagged":
-        return(CheckOnline());
+        return(IsFolderSelected() && MailOfflineMgr.isOnline());
       case "cmd_downloadSelected":
-        return (IsFolderSelected() && CheckOnline() && GetNumSelectedMessages() > 0);
+        return (IsFolderSelected() && MailOfflineMgr.isOnline() && GetNumSelectedMessages() > 0);
       case "cmd_synchronizeOffline":
-        return CheckOnline() && IsAccountOfflineEnabled();       
+        return MailOfflineMgr.isOnline() && IsAccountOfflineEnabled();
       case "cmd_settingsOffline":
         return IsAccountOfflineEnabled();
+      case "cmd_moveToFolderAgain":
+        return (pref.getCharPref("mail.last_msg_movecopy_target_uri") && GetNumSelectedMessages() > 0);
       default:
         return false;
     }
@@ -396,7 +400,7 @@ var DefaultController =
     // if the user invoked a key short cut then it is possible that we got here for a command which is
     // really disabled. kick out if the command should be disabled.
     if (!this.isCommandEnabled(command)) return;
-   
+
 		switch ( command )
 		{
 			case "cmd_close":
@@ -438,7 +442,7 @@ var DefaultController =
 				break;
       case "cmd_createFilterFromMenu":
         MsgCreateFilter();
-        break;        
+        break;
       case "cmd_createFilterFromPopup":
         break;// This does nothing because the createfilter is invoked from the popupnode oncommand.
 			case "button_delete":
@@ -486,6 +490,14 @@ var DefaultController =
 			case "cmd_previousFlaggedMsg":
 				MsgPreviousFlaggedMessage();
 				break;
+      case "button_goForward":
+      case "cmd_goForward":
+        MsgGoForward();
+        break;
+      case "button_goBack":
+      case "cmd_goBack":
+        MsgGoBack();
+        break;
       case "cmd_goStartPage":
         HideMessageHeaderPane();
         loadStartPage();
@@ -504,10 +516,10 @@ var DefaultController =
 				messenger.Redo(msgWindow);
 				break;
 			case "cmd_expandAllThreads":
-                gDBView.doCommand(nsMsgViewCommandType.expandAll);
+        gDBView.doCommand(nsMsgViewCommandType.expandAll);
 				break;
 			case "cmd_collapseAllThreads":
-                gDBView.doCommand(nsMsgViewCommandType.collapseAll);
+        gDBView.doCommand(nsMsgViewCommandType.collapseAll);
 				break;
 			case "cmd_renameFolder":
 				MsgRenameFolder();
@@ -516,7 +528,7 @@ var DefaultController =
 				MsgSendUnsentMsgs();
 				return;
 			case "cmd_openMessage":
-                MsgOpenSelectedMessages();
+        MsgOpenSelectedMessages();
 				return;
 			case "cmd_printSetup":
 			  NSPrintSetup();
@@ -545,19 +557,19 @@ var DefaultController =
 			case "cmd_find":
         // make sure the message pane has focus before we start a find since we only support searching
         // within the message body. Do it here and not in MsgFind() which can be called from standalone where we don't want to set focus
-        SetFocusMessagePane(); 
+        SetFocusMessagePane();
 				MsgFind();
 				return;
 			case "cmd_findAgain":
         // make sure the message pane has focus before we start a find since we only support searching
         // within the message body. Do it here and not in MsgFind() which can be called from standalone where we don't want to set focus
-        SetFocusMessagePane(); 
+        SetFocusMessagePane();
 				MsgFindAgain(false);
 				return;
-			case "cmd_findPrev":
+			case "cmd_findPrevious":
         // make sure the message pane has focus before we start a find since we only support searching
         // within the message body. Do it here and not in MsgFind() which can be called from standalone where we don't want to set focus
-        SetFocusMessagePane(); 
+        SetFocusMessagePane();
 				MsgFindAgain(true);
 				return;
       case "cmd_markReadByDate":
@@ -578,7 +590,7 @@ var DefaultController =
 				return;
 			case "cmd_markAllRead":
         gDBView.doCommand(nsMsgViewCommandType.markAllRead);
-				return;       
+				return;
       case "button_junk":
         MsgJunk();
         return;
@@ -601,71 +613,60 @@ var DefaultController =
         MsgApplyFilters(null);
         return;
       case "cmd_runJunkControls":
-        analyzeFolderForJunk();
+        filterFolderForJunk();
         return;
       case "cmd_deleteJunk":
         deleteJunkInFolder();
         return;
-      case "cmd_label0":
-        gDBView.doCommand(nsMsgViewCommandType.label0);
- 				return;
-      case "cmd_label1":
-        gDBView.doCommand(nsMsgViewCommandType.label1);
-        return; 
-      case "cmd_label2":
-        gDBView.doCommand(nsMsgViewCommandType.label2);
-        return; 
-      case "cmd_label3":
-        gDBView.doCommand(nsMsgViewCommandType.label3);
-        return; 
-      case "cmd_label4":
-        gDBView.doCommand(nsMsgViewCommandType.label4);
-        return; 
-      case "cmd_label5":
-        gDBView.doCommand(nsMsgViewCommandType.label5);
-        return; 
 			case "cmd_emptyTrash":
 				MsgEmptyTrash();
 				return;
 			case "cmd_compactFolder":
 				MsgCompactFolder(true);
 				return;
-            case "cmd_downloadFlagged":
-                MsgDownloadFlagged();
-                break;
-            case "cmd_downloadSelected":
-                MsgDownloadSelected();
-                break;
-            case "cmd_synchronizeOffline":
-                MsgSynchronizeOffline();
-                break;
-            case "cmd_settingsOffline":
-                MsgSettingsOffline();
-                break;
-            case "cmd_selectAll":
-                // move the focus so the user can delete the newly selected messages, not the folder
-                SetFocusThreadPane();
-                // if in threaded mode, the view will expand all before selecting all
-                gDBView.doCommand(nsMsgViewCommandType.selectAll)
-                if (gDBView.numSelected != 1) {
-                    setTitleFromFolder(gDBView.msgFolder,null);
-                    ClearMessagePane();
-                }
-                break;
-            case "cmd_selectThread":
-                gDBView.doCommand(nsMsgViewCommandType.selectThread);
-                break;
+      case "cmd_downloadFlagged":
+          MsgDownloadFlagged();
+          break;
+      case "cmd_downloadSelected":
+          MsgDownloadSelected();
+          break;
+      case "cmd_synchronizeOffline":
+          MsgSynchronizeOffline();
+          break;
+      case "cmd_settingsOffline":
+          MailOfflineMgr.openOfflineAccountSettings();
+          break;
+      case "cmd_moveToFolderAgain":
+          var folderId = pref.getCharPref("mail.last_msg_movecopy_target_uri");
+          if (pref.getBoolPref("mail.last_msg_movecopy_was_move"))
+            MsgMoveMessage(folderId);
+          else
+            MsgCopyMessage(folderId);
+          break;
+      case "cmd_selectAll":
+          // move the focus so the user can delete the newly selected messages, not the folder
+          SetFocusThreadPane();
+          // if in threaded mode, the view will expand all before selecting all
+          gDBView.doCommand(nsMsgViewCommandType.selectAll)
+          if (gDBView.numSelected != 1) {
+              setTitleFromFolder(gDBView.msgFolder,null);
+              ClearMessagePane();
+          }
+          break;
+      case "cmd_selectThread":
+          gDBView.doCommand(nsMsgViewCommandType.selectThread);
+          break;
 		}
 	},
-	
+
 	onEvent: function(event)
 	{
 		// on blur events set the menu item texts back to the normal values
 		if ( event == 'blur' )
-        {
-            goSetMenuValue('cmd_undo', 'valueDefault');
-            goSetMenuValue('cmd_redo', 'valueDefault');
-        }
+    {
+      goSetMenuValue('cmd_undo', 'valueDefault');
+      goSetMenuValue('cmd_redo', 'valueDefault');
+    }
 	}
 };
 
@@ -686,17 +687,16 @@ function FocusRingUpdate_Mail()
   // WhichPaneHasFocus() uses on top.document.commandDispatcher.focusedElement
   // to determine which pane has focus
   // if the focusedElement is null, we're here on a blur.
-  // nsFocusController::Blur() calls nsFocusController::SetFocusedElement(null), 
+  // nsFocusController::Blur() calls nsFocusController::SetFocusedElement(null),
   // which will update any commands listening for "focus".
-  // we really only care about nsFocusController::Focus() happens, 
+  // we really only care about nsFocusController::Focus() happens,
   // which calls nsFocusController::SetFocusedElement(element)
   var currentFocusedElement = WhichPaneHasFocus();
-  if (!currentFocusedElement)
-    return;
-      
-	if (currentFocusedElement != gLastFocusedElement) {
-    currentFocusedElement.setAttribute("focusring", "true");
-    
+
+  if (currentFocusedElement != gLastFocusedElement) {
+    if (currentFocusedElement)
+      currentFocusedElement.setAttribute("focusring", "true");
+
     if (gLastFocusedElement)
       gLastFocusedElement.removeAttribute("focusring");
 
@@ -713,36 +713,34 @@ function FocusRingUpdate_Mail()
 function WhichPaneHasFocus()
 {
   var threadTree = GetThreadTree();
-  var searchInput = GetSearchInput();
   var folderTree = GetFolderTree();
   var messagePane = GetMessagePane();
-    
+
   if (top.document.commandDispatcher.focusedWindow == GetMessagePaneFrame())
     return messagePane;
 
-	var currentNode = top.document.commandDispatcher.focusedElement;	
+	var currentNode = top.document.commandDispatcher.focusedElement;
 	while (currentNode) {
     if (currentNode === threadTree ||
-        currentNode === searchInput || 
         currentNode === folderTree ||
         currentNode === messagePane)
       return currentNode;
-    			
+
 		currentNode = currentNode.parentNode;
   }
-	
+
 	return null;
 }
 
 function SetupCommandUpdateHandlers()
 {
 	var widget;
-	
+
 	// folder pane
 	widget = GetFolderTree();
 	if ( widget )
 		widget.controllers.appendController(FolderPaneController);
-	
+
 	top.controllers.insertControllerAt(0, DefaultController);
 }
 
@@ -809,6 +807,8 @@ function IsRenameFolderEnabled()
 function IsCanSearchMessagesEnabled()
 {
   var folderURI = GetSelectedFolderURI();
+  if (!folderURI)
+    return false;
   var server = GetServer(folderURI);
   return server.canSearchMessages;
 }
@@ -819,7 +819,7 @@ function IsFolderCharsetEnabled()
 
 function IsPropertiesEnabled(command)
 {
-   try 
+   try
    {
       var folderTree = GetFolderTree();
       var folderResource = GetSelectedFolderResource();
@@ -828,10 +828,10 @@ function IsPropertiesEnabled(command)
       // it should be "Edit | Properties..."
       if (GetFolderAttribute(folderTree, folderResource, "IsServer") == "true")
         goSetMenuValue(command, "valueGeneric");
-      else 
+      else
         goSetMenuValue(command, isNewsURI(folderResource.Value) ? "valueNewsgroup" : "valueFolder");
    }
-   catch (ex) 
+   catch (ex)
    {
       // properties menu failure
    }
@@ -894,8 +894,8 @@ function MsgDeleteFolder()
             var protocolInfo = Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + selectedFolder.server.type].getService(Components.interfaces.nsIMsgProtocolInfo);
 
             // do not allow deletion of special folders on imap accounts
-            if ((specialFolder == "Sent" || 
-                specialFolder == "Drafts" || 
+            if ((specialFolder == "Sent" ||
+                specialFolder == "Drafts" ||
                 specialFolder == "Templates" ||
                 (specialFolder == "Junk" && !CanRenameDeleteJunkMail(GetSelectedFolderURI()))) &&
                 !protocolInfo.specialFoldersDeletionAllowed)
@@ -905,7 +905,7 @@ function MsgDeleteFolder()
                 var specialFolderDeletionErrTitle = gMessengerBundle.getString("specialFolderDeletionErrTitle");
                 promptService.alert(window, specialFolderDeletionErrTitle, errorMessage);
                 continue;
-            }   
+            }
             else if (isNewsURI(folderResource.Value))
             {
                 var unsubscribe = ConfirmUnsubscribe(selectedFolder);
@@ -947,9 +947,7 @@ function MsgNextFlaggedMessage()
 
 function MsgNextUnreadThread()
 {
-	//First mark the current thread as read.  Then go to the next one.
-	MsgMarkThreadAsRead();
-	GoNextMessage(nsMsgNavigationType.nextUnreadThread, true);
+  GoNextMessage(nsMsgNavigationType.nextUnreadThread, true);
 }
 
 function MsgPreviousMessage()
@@ -967,6 +965,16 @@ function MsgPreviousFlaggedMessage()
 	GoNextMessage(nsMsgNavigationType.previousFlagged, true);
 }
 
+function MsgGoForward()
+{
+  GoNextMessage(nsMsgNavigationType.forward, true);
+}
+
+function MsgGoBack()
+{
+  GoNextMessage(nsMsgNavigationType.back, true);
+}
+
 function GetFolderNameFromUri(uri, tree)
 {
 	var folderResource = RDF.GetResource(uri);
@@ -987,65 +995,33 @@ function GetFolderNameFromUri(uri, tree)
 	return nameResult.Value;
 }
 
-/* XXX hiding the search bar while it is focus kills the keyboard so we focus the thread pane */
-function SearchBarToggled()
-{
-  var searchBox = document.getElementById('searchBox');
-  if (searchBox)
-  {
-    var attribValue = searchBox.getAttribute("hidden") ;
-    if (attribValue == "true")
-    {
-      /*come out of quick search view */
-      if (gDBView && gDBView.viewType == nsMsgViewType.eShowQuickSearchResults)
-        onClearSearch();
-    }
-    else
-    {
-      /*we have to initialize searchInput because we cannot do it when searchBox is hidden */
-      var searchInput = GetSearchInput();
-      searchInput.value="";
-    }
-  }
-
-  for (var currentNode = top.document.commandDispatcher.focusedElement; currentNode; currentNode = currentNode.parentNode) {
-    // But skip the last node, which is a XULDocument.
-    if ((currentNode instanceof XULElement) && currentNode.hidden) {
-      SetFocusThreadPane();
-      return;
-    }
-  }
-}
-
 function SwitchPaneFocus(event)
 {
-  var focusedElement = WhichPaneHasFocus();
   var folderTree = GetFolderTree();
   var threadTree = GetThreadTree();
-  var searchInput = GetSearchInput();
   var messagePane = GetMessagePane();
 
+  var focusedElement = WhichPaneHasFocus();
+  if (focusedElement == null)       // focus not on one of the main three panes (probably toolbar)
+    focusedElement = threadTree;    // treat as if on thread tree
+
   if (event && event.shiftKey)
   {
-    if (focusedElement == threadTree && searchInput.parentNode.getAttribute('hidden') != 'true')
-      searchInput.focus();
-    else if ((focusedElement == threadTree || focusedElement == searchInput) && !IsFolderPaneCollapsed())
+    // Reverse traversal: Message -> Thread -> Folder -> Message
+    if (focusedElement == threadTree && !IsFolderPaneCollapsed())
       folderTree.focus();
     else if (focusedElement != messagePane && !IsMessagePaneCollapsed())
       SetFocusMessagePane();
-    else 
+    else
       threadTree.focus();
   }
   else
   {
-    if (focusedElement == searchInput)
-      threadTree.focus();
-    else if (focusedElement == threadTree && !IsMessagePaneCollapsed())
+    // Forward traversal: Folder -> Thread -> Message -> Folder
+    if (focusedElement == threadTree && !IsMessagePaneCollapsed())
       SetFocusMessagePane();
     else if (focusedElement != folderTree && !IsFolderPaneCollapsed())
       folderTree.focus();
-    else if (searchInput.parentNode.getAttribute('hidden') != 'true')
-      searchInput.focus();
     else
       threadTree.focus();
   }
@@ -1065,11 +1041,11 @@ function SetFocusThreadPane()
 
 function SetFocusMessagePane()
 {
-    var messagePaneFrame = GetMessagePaneFrame();
-    messagePaneFrame.focus();
+  // messagePaneFrame.focus() fails to blur the currently focused element
+  document.commandDispatcher.advanceFocusIntoSubtree(GetMessagePane());
 }
 
-function is_collapsed(element) 
+function is_collapsed(element)
 {
   return (element.getAttribute('state') == 'collapsed');
 }
