@@ -1,4 +1,4 @@
-//@line 38 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/base/content/msgHdrViewOverlay.js"
+//@line 38 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/base/content/msgHdrViewOverlay.js"
 
 
 /* This is where functions related to displaying the headers for a selected message in the
@@ -23,19 +23,13 @@ const kLargeIcon = 32;
 const kSmallIcon = 16;
 
 var gViewAllHeaders = false;
-var gNumAddressesToShow = 3;
 var gShowOrganization = false;
 var gShowLargeAttachmentView = false;
 var gShowUserAgent = false;
+var gExtraExpandedHeaders;
 var gMinNumberOfHeaders = 0;
 var gDummyHeaderIdIndex = 0;
-var gCollectIncoming = false;
-var gCollectOutgoing = false;
-var gCollectNewsgroup = false;
 var gCollapsedHeaderViewMode = false;
-var gSlimAttachmentView = false;
-var gCollectAddressTimer = null;
-var gCollectAddress = null;
 var gBuildAttachmentsForCurrentMsg = false;
 var gBuildAttachmentPopupForCurrentMsg = true;
 var gBuiltExpandedView = false;
@@ -51,7 +45,6 @@ var gDeleteLabelAccesskey;
 var gMessengerBundle;
 var gProfileDirURL;
 var gIOService;
-var gFileHandler;  
 var gShowCondensedEmailAddresses = true; // show the friendly display names for people I know instead of the name + email address
 var gPersonalAddressBookDirectory; // used for determining if we want to show just the display name in email address nodes
 
@@ -85,21 +78,22 @@ var gMessageListeners = new Array;
 // Our first view is the collapsed view. This is very light weight view of the data. We only show a couple
 // fields.
 var gCollapsedHeaderList = [ {name:"subject", outputFunction:updateHeaderValueInTextNode},
-                             {name:"from", useShortView:true, outputFunction:OutputEmailAddresses},
+                             {name:"from", useToggle:true, useShortView:true, outputFunction:OutputEmailAddresses},
                              {name:"date", outputFunction:updateHeaderValueInTextNode}];
 
 // We also have an expanded header view. This shows many of your more common (and useful) headers.
 var gExpandedHeaderList = [ {name:"subject"}, 
-                            {name:"from", outputFunction:OutputEmailAddresses},
+                            {name:"from", useToggle:true, outputFunction:OutputEmailAddresses},
                             {name:"sender", outputFunction:OutputEmailAddresses},
-                            {name:"reply-to", outputFunction:OutputEmailAddresses},
+                            {name:"reply-to", useToggle:true, outputFunction:OutputEmailAddresses},
                             {name:"date"},
                             {name:"to", useToggle:true, outputFunction:OutputEmailAddresses},
                             {name:"cc", useToggle:true, outputFunction:OutputEmailAddresses},
                             {name:"bcc", useToggle:true, outputFunction:OutputEmailAddresses},
                             {name:"newsgroups", outputFunction:OutputNewsgroups},
                             {name:"followup-to", outputFunction:OutputNewsgroups},
-                            {name:"content-base"} ];
+                            {name:"content-base"},
+                            {name:"tags"} ];
 
 // Now, for each view the message pane can generate, we need a global table of headerEntries. These
 // header entry objects are generated dynamically based on the static date in the header lists (see above)
@@ -131,11 +125,21 @@ var currentAttachments = new Array();
 // headerListInfo --> entry from a header list.
 function createHeaderEntry(prefix, headerListInfo)
 {
+  var useShortView = false;
   var partialIDName = prefix + headerListInfo.name;
   this.enclosingBox = document.getElementById(partialIDName + 'Box');
   this.textNode = document.getElementById(partialIDName + 'Value');
   this.isValid = false;
 
+  if ("useShortView" in headerListInfo)
+  {
+    useShortView = headerListInfo.useShortView;
+    if (useShortView)
+      this.enclosingBox = this.textNode;
+    else
+      this.enclosingBox.emailAddressNode = this.textNode;
+  }
+
   if ("useToggle" in headerListInfo)
   {
     this.useToggle = headerListInfo.useToggle;
@@ -149,13 +153,8 @@ function createHeaderEntry(prefix, headerListInfo)
   else
    this.useToggle = false;
 
-  if ("useShortView" in headerListInfo)
-  {
-    this.useShortView = headerListInfo.useShortView;
-    this.enclosingBox.emailAddressNode = this.textNode;
-  }
-  else
-    this.useShortView = false;
+  if (this.textNode)
+    this.textNode.useShortView = useShortView;
 
   if ("outputFunction" in headerListInfo)
     this.outputFunction = headerListInfo.outputFunction;
@@ -180,6 +179,12 @@ function initializeHeaderViewTables()
       gExpandedHeaderView[headerName] = new createHeaderEntry('expanded', gExpandedHeaderList[index]);
     }
     
+    var extraHeaders = gExtraExpandedHeaders.split(' ');
+    for (index = 0; index < extraHeaders.length; index++)
+    {
+      var extraHeader = extraHeaders[index];
+      gExpandedHeaderView[extraHeader.toLowerCase()] = new createNewHeaderView(extraHeader, extraHeader + ':');
+    }
     if (gShowOrganization)
     {
       var organizationEntry = {name:"organization", outputFunction:updateHeaderValue};
@@ -201,25 +206,20 @@ function OnLoadMsgHeaderPane()
   
   // load any preferences that at are global with regards to 
   // displaying a message...
-  gNumAddressesToShow = pref.getIntPref("mailnews.max_header_display_length");
-  gCollectIncoming = pref.getBoolPref("mail.collect_email_address_incoming");
-  gCollectNewsgroup = pref.getBoolPref("mail.collect_email_address_newsgroup");
-  gCollectOutgoing = pref.getBoolPref("mail.collect_email_address_outgoing");
   gShowUserAgent = pref.getBoolPref("mailnews.headers.showUserAgent");
   gMinNumberOfHeaders = pref.getIntPref("mailnews.headers.minNumHeaders");
   gShowOrganization = pref.getBoolPref("mailnews.headers.showOrganization");
   gShowLargeAttachmentView = pref.getBoolPref("mailnews.attachments.display.largeView");
   gShowCondensedEmailAddresses = pref.getBoolPref("mail.showCondensedAddresses");
+  gExtraExpandedHeaders = pref.getCharPref("mailnews.headers.extraExpandedHeaders");
 
   // listen to the 
   pref.addObserver("mail.showCondensedAddresses", MsgHdrViewObserver, false);
 
   initializeHeaderViewTables();
 
-  var toggleHeaderView = document.getElementById("msgHeaderView");
-  var initialCollapsedSetting = toggleHeaderView.getAttribute("state");
-  if (initialCollapsedSetting == "true")
-    gCollapsedHeaderViewMode = true;   
+  var deckHeaderView = document.getElementById("msgHeaderViewDeck");
+  gCollapsedHeaderViewMode = deckHeaderView.selectedIndex == 0;   
 
   // dispatch an event letting any listeners know that we have loaded the message pane
   var event = document.createEvent('Events');
@@ -262,14 +262,6 @@ var messageHeaderSink = {
     onStartHeaders: function()
     {
       this.mSaveHdr = null;
-      // clear out any pending collected address timers...
-      if (gCollectAddressTimer)
-      {
-        gCollectAddress = "";        
-        clearTimeout(gCollectAddressTimer);
-        gCollectAddressTimer = null;
-      }
-
       // every time we start to redisplay a message, check the view all headers pref....
       var showAllHeadersPref = pref.getIntPref("mail.show_headers");
       if (showAllHeadersPref == 2)
@@ -314,17 +306,17 @@ var messageHeaderSink = {
       
       ShowMessageHeaderPane();
       UpdateMessageHeaders();
-      if (gIsEditableMsgFolder)
-        ShowEditMessageButton();
+      ShowEditMessageButton();
       
       for (index in gMessageListeners)
         gMessageListeners[index].onEndHeaders();
     },
+    
     processHeaders: function(headerNameEnumerator, headerValueEnumerator, dontCollectAddress)
     {
       this.onStartHeaders(); 
-      var fromMailbox;
 
+      const kMailboxSeparator = ", ";
       var index = 0;
       while (headerNameEnumerator.hasMore()) 
       {
@@ -340,12 +332,7 @@ var messageHeaderSink = {
         // already. 
         if (lowerCaseHeaderName == "x-mailer" || lowerCaseHeaderName == "x-mimeole")
           lowerCaseHeaderName = "user-agent";   
-        else if (lowerCaseHeaderName == "sender" && msgHeaderParser &&
-            header.headerValue && fromMailbox)
-        {
-          if (msgHeaderParser.extractHeaderAddressMailboxes(null, header.headerValue) == fromMailbox)
-            continue;
-        }
+
         if (this.mDummyMsgHeader)
         {
           if (lowerCaseHeaderName == "from")
@@ -378,39 +365,21 @@ var messageHeaderSink = {
         }
         else
          currentHeaderData[lowerCaseHeaderName] = header;
-        if (lowerCaseHeaderName == "from")
-        {
-          if (msgHeaderParser && header.headerValue)
-            fromMailbox = msgHeaderParser.extractHeaderAddressMailboxes(null, header.headerValue);
-
-          if (header.headerValue) {
-            try
-            {
-              if ((gCollectIncoming && !dontCollectAddress) || 
-                  (gCollectNewsgroup && dontCollectAddress))
-              {
-                if (!abAddressCollector)
-                  abAddressCollector = Components.classes[abAddressCollectorContractID].getService(Components.interfaces.nsIAbAddressCollecter);
-
-                gCollectAddress = header.headerValue;
-                // collect, and add card if doesn't exist, unknown preferred send format
-                gCollectAddressTimer = setTimeout('abAddressCollector.collectUnicodeAddress(gCollectAddress, true, Components.interfaces.nsIAbPreferMailFormat.unknown);', 2000);
-              }
-              else if (gCollectOutgoing) 
-              {
-                if (!abAddressCollector)
-                  abAddressCollector = Components.classes[abAddressCollectorContractID].getService(Components.interfaces.nsIAbAddressCollecter);
-
-                // collect, but only update existing cards, unknown preferred send format
-                gCollectAddress = header.headerValue;
-                gCollectAddressTimer = setTimeout('abAddressCollector.collectUnicodeAddress(gCollectAddress, false, Components.interfaces.nsIAbPreferMailFormat.unknown);', 2000);
-              }
-            }
-            catch(ex) {}
-          } 
-        } // if lowerCaseHeaderName == "from"
       } // while we have more headers to parse
 
+      // process message tags as if they were headers in the message
+      SetTagHeader();      
+
+      if (("from" in currentHeaderData) && ("sender" in currentHeaderData) && msgHeaderParser)
+      {
+        var senderMailbox = kMailboxSeparator + msgHeaderParser.extractHeaderAddressMailboxes(null,
+                            currentHeaderData.sender.headerValue) + kMailboxSeparator;
+        var fromMailboxes = kMailboxSeparator + msgHeaderParser.extractHeaderAddressMailboxes(null,
+                            currentHeaderData.from.headerValue) + kMailboxSeparator;
+        if (fromMailboxes.indexOf(senderMailbox) >= 0)
+          delete currentHeaderData.sender;
+      }
+
       this.onEndHeaders();
     },
 
@@ -437,7 +406,7 @@ var messageHeaderSink = {
 
       // display name optimization. Eliminate any large quantities of white space from the display name.
       // such that Hello       World.txt becomes Hello World.txt.
-      var displayName = displayName.replace(/ +/g, " ");
+      displayName = displayName.replace(/ +/g, " ");
           
       currentAttachments.push (new createNewAttachmentInfo(contentType, url, displayName, uri, isExternalAttachment));
       // if we have an attachment, set the MSG_FLAG_ATTACH flag on the hdr
@@ -496,11 +465,11 @@ var messageHeaderSink = {
 
     mSecurityInfo  : null,
     mSaveHdr: null,
-    getSecurityInfo: function()
+    get securityInfo()
     {
       return this.mSecurityInfo;
     },
-    setSecurityInfo: function(aSecurityInfo)
+    set securityInfo(aSecurityInfo)
     {
       this.mSecurityInfo = aSecurityInfo;
     },
@@ -512,9 +481,65 @@ var messageHeaderSink = {
       if (!this.mDummyMsgHeader)
         this.mDummyMsgHeader = new nsDummyMsgHeader();
       return this.mDummyMsgHeader;
+    },
+    mProperties: null,
+    get properties()
+    {
+      if (!this.mProperties)
+        this.mProperties = Components.classes["@mozilla.org/hash-property-bag;1"].
+          createInstance(Components.interfaces.nsIWritablePropertyBag2);
+      return this.mProperties;
     }
 };
 
+function SetTagHeader()
+{
+  // it would be nice if we passed in the msgHdr from the back end
+  var msgHdr;
+  try
+  {
+    msgHdr = gDBView.hdrForFirstSelectedMessage;
+  }
+  catch (ex)
+  {
+    return; // no msgHdr to add our tags to
+  }
+
+  // get the list of known tags
+  var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
+                   .getService(Components.interfaces.nsIMsgTagService);
+  var tagArray = tagService.getAllTags({});
+  var tagKeys = {};
+  for each (var tagInfo in tagArray)
+    if (tagInfo.tag)
+      tagKeys[tagInfo.key] = true;
+
+  // extract the tag keys from the msgHdr
+  var msgKeyArray = msgHdr.getStringProperty("keywords").split(" ");
+
+  // attach legacy label to the front if not already there
+  var label = msgHdr.label;
+  if (label)
+  {
+    var labelKey = "$label" + label;
+    if (msgKeyArray.indexOf(labelKey) < 0)
+      msgKeyArray.unshift(labelKey);
+  }
+
+ // Rebuild the keywords string with just the keys that are actual tags or
+  // legacy labels and not other keywords like Junk and NonJunk.
+  // Retain their order, though, with the label as oldest element.
+  for (var i = msgKeyArray.length - 1; i >= 0; --i)
+    if (!(msgKeyArray[i] in tagKeys))
+      msgKeyArray.splice(i, 1); // remove non-tag key
+  var msgKeys = msgKeyArray.join(" ");
+
+  if (msgKeys)
+    currentHeaderData.tags = {headerName: "tags", headerValue: msgKeys};
+  else // no more tags, so clear out the header field
+    delete currentHeaderData.tags;
+}
+
 function EnsureSubjectValue()
 {
   if (!('subject' in currentHeaderData))
@@ -532,6 +557,29 @@ function CheckNotify()
     NotifyClearAddresses();
 }
 
+function OnTagsChange()
+{
+  // rebuild the tag headers
+  SetTagHeader();
+
+  // now update the expanded header view to rebuild the tags,
+  // and then show or hide the tag header box.  
+  if (gBuiltExpandedView)
+  {
+    var headerEntry = gExpandedHeaderView.tags;
+    if (headerEntry)
+    {
+      headerEntry.valid = ("tags" in currentHeaderData);
+      if (headerEntry.valid)
+        headerEntry.outputFunction(headerEntry, currentHeaderData.tags.headerValue);
+      
+      // if we are showing the expanded header view then we may need to collapse or
+      // show the tag header box...
+      if (!gCollapsedHeaderViewMode)
+        headerEntry.enclosingBox.collapsed = !headerEntry.valid;
+    }
+  }
+}
 
 // flush out any local state being held by a header entry for a given
 // table
@@ -608,7 +656,7 @@ function EnsureMinimumNumberOfHeaders (headerTable)
     while (numEmptyHeaders)
     {
       var dummyHeaderId = "Dummy-Header" + gDummyHeaderIdIndex;
-      gExpandedHeaderView[dummyHeaderId] = new createNewHeaderView(dummyHeaderId);
+      gExpandedHeaderView[dummyHeaderId] = new createNewHeaderView(dummyHeaderId, "");
       gExpandedHeaderView[dummyHeaderId].valid = true;
 
       gDummyHeaderIdIndex++;
@@ -636,39 +684,19 @@ function updateHeaderViews()
 
 function ToggleHeaderView ()
 {
-  var expandedNode = document.getElementById("expandedHeaderView");
-  var collapsedNode = document.getElementById("collapsedHeaderView");
-  var toggleHeaderView = document.getElementById("msgHeaderView");
+  gCollapsedHeaderViewMode = !gCollapsedHeaderViewMode;
+  // Work around a xul deck bug where the height of the deck is determined by the tallest panel in the deck
+  // even if that panel is not selected...
+  document.getElementById('msgHeaderViewDeck').selectedPanel.collapsed = true;
+  
+  UpdateMessageHeaders(); 
+  
+  // select the new panel. 
+  document.getElementById('msgHeaderViewDeck').selectedIndex = gCollapsedHeaderViewMode ? 0 : 1;
 
-  if (gCollapsedHeaderViewMode)
-  {          
-    gCollapsedHeaderViewMode = false;
-    // hide the current view
-    hideHeaderView(gCollapsedHeaderView);
-    // update the current view
-    UpdateMessageHeaders();
-    
-    // now uncollapse / collapse the right views
-    expandedNode.collapsed = false;
-    collapsedNode.collapsed = true;
-  }
-  else
-  {
-    gCollapsedHeaderViewMode = true;
-    // hide the current view
-    hideHeaderView(gExpandedHeaderView);
-    // update the current view
-    UpdateMessageHeaders();
-    
-    // now uncollapse / collapse the right views
-    collapsedNode.collapsed = false;
-    expandedNode.collapsed = true;
-  }  
-
-  if (gCollapsedHeaderViewMode)
-    toggleHeaderView.setAttribute("state", "true");
-  else
-    toggleHeaderView.setAttribute("state", "false");
+  // Work around a xul deck bug where the height of the deck is determined by the tallest panel in the deck
+  // even if that panel is not selected...  
+  document.getElementById('msgHeaderViewDeck').selectedPanel.collapsed = false;
 }
 
 // default method for updating a header value into a header entry
@@ -682,23 +710,13 @@ function updateHeaderValueInTextNode(headerEntry, headerValue)
   headerEntry.textNode.value = headerValue;
 }
 
-function createNewHeaderView(headerName)
+function createNewHeaderView(headerName, label)
 {
   var idName = 'expanded' + headerName + 'Box';
   var newHeader = document.createElement("mail-headerfield");
-  newHeader.setAttribute('id', idName);
-
-  if (headerName.indexOf("Dummy-Header") == 0) // -1 means not found, 0 means starts at the beginning
-  {
-    newHeader.setAttribute('label', "");
-  } 
-  else
-  {
-    newHeader.setAttribute('label', currentHeaderData[headerName].headerName + ':');
-    // all mail-headerfield elements are keyword related
-    newHeader.setAttribute('keywordrelated','true');
-  }
   
+  newHeader.setAttribute('id', idName);
+  newHeader.setAttribute('label', label);
   newHeader.collapsed = true;
 
   // this new element needs to be inserted into the view...
@@ -709,7 +727,6 @@ function createNewHeaderView(headerName)
   this.enclosingBox = newHeader;
   this.isValid = false;
   this.useToggle = false;
-  this.useShortView = false;
   this.outputFunction = updateHeaderValue;
 }
 
@@ -721,6 +738,12 @@ function UpdateMessageHeaders()
   // iterate over each header we received and see if we have a matching entry in each
   // header view table...
 
+  var headerName;
+
+  // Remove the height attr so that it redraws correctly. Works around a problem that
+  // attachment-splitter causes if it's moved high enough to affect the header box:
+  document.getElementById('msgHeaderView').removeAttribute('height');
+
   for (headerName in currentHeaderData)
   {
     var headerField = currentHeaderData[headerName];
@@ -753,7 +776,7 @@ function UpdateMessageHeaders()
       {
         // for view all headers, if we don't have a header field for this value....cheat and create one....then
         // fill in a headerEntry
-        gExpandedHeaderView[headerName] = new createNewHeaderView(headerName);
+        gExpandedHeaderView[headerName] = new createNewHeaderView(headerName, currentHeaderData[headerName].headerName + ':');
         headerEntry = gExpandedHeaderView[headerName];
       }
     } // if we are in expanded view....
@@ -785,19 +808,7 @@ function ClearCurrentHeaders()
 
 function ShowMessageHeaderPane()
 { 
-  var node;
-  if (gCollapsedHeaderViewMode)
-  {          
-    node = document.getElementById("collapsedHeaderView");
-    if (node)
-      node.collapsed = false;
-  }
-  else
-  {
-    node = document.getElementById("expandedHeaderView");
-    if (node)
-      node.collapsed = false;
-  }
+  document.getElementById('msgHeaderView').collapsed = false;
 
 	/* workaround for 39655 */
   if (gFolderJustSwitched) 
@@ -810,22 +821,13 @@ function ShowMessageHeaderPane()
 
 function HideMessageHeaderPane()
 {
-  var node = document.getElementById("collapsedHeaderView");
-  if (node)
-    node.collapsed = true;
+  document.getElementById('msgHeaderView').collapsed = true;
 
-  node = document.getElementById("expandedHeaderView");
-  if (node)
-    node.collapsed = true;
-
-  // we also have to disable the File/Attachments menuitem
-  node = document.getElementById("fileAttachmentMenu");
-  if (node)
-    node.setAttribute("disabled", "true");
-
-  node = document.getElementById("attachmentView");
-  if (node)
-    node.collapsed = true;
+  // disable the File/Attachments menuitem
+  document.getElementById("fileAttachmentMenu").setAttribute("disabled", "true");
+  // disable the attachment box
+  document.getElementById("attachmentView").collapsed = true;
+  document.getElementById("attachment-splitter").collapsed = true;
 }
 
 function OutputNewsgroups(headerEntry, headerValue)
@@ -838,17 +840,12 @@ function OutputNewsgroups(headerEntry, headerValue)
 // extracts them one by one, linkifying each email address into a mailto url. 
 // Then we add the link-ified email address to the parentDiv passed in.
 // 
-// defaultParentDiv --> the div to add the link-ified email addresses into. 
 // emailAddresses --> comma separated list of the addresses for this header field
-// includeShortLongToggle --> true if you want to include the ability to toggle between short/long
-// address views for this header field. If true, then pass in a another div which is the div the long
-// view will be added too...
-// useShortView --> if true, we'll only generate the Name of the email address field instead of
-//                        showing the name + the email address.
 
 function OutputEmailAddresses(headerEntry, emailAddresses)
 {
-  if ( !emailAddresses ) return;
+  if (!emailAddresses)
+    return;
 
   if (msgHeaderParser)
   {
@@ -864,93 +861,31 @@ function OutputEmailAddresses(headerEntry, emailAddresses)
       // if we want to include short/long toggle views and we have a long view, always add it.
       // if we aren't including a short/long view OR if we are and we haven't parsed enough
       // addresses to reach the cutoff valve yet then add it to the default (short) div.
+      var address = {};
+      address.emailAddress = addresses.value[index];
+      address.fullAddress = fullNames.value[index];
+      address.displayName = names.value[index];
       if (headerEntry.useToggle)
-      {
-        var address = {};
-        address.emailAddress = addresses.value[index];
-        address.fullAddress = fullNames.value[index];
-        address.displayName = names.value[index];
         headerEntry.enclosingBox.addAddressView(address);
-      }
       else
-      {
-        updateEmailAddressNode(headerEntry.enclosingBox.emailAddressNode, addresses.value[index], 
-                               fullNames.value[index], names.value[index], headerEntry.useShortView);
-      }
-
-      if (headerEntry.enclosingBox.getAttribute("id") == "expandedfromBox") {
-        setFromBuddyIcon(addresses.value[index]);
-      }
-
+        updateEmailAddressNode(headerEntry.enclosingBox.emailAddressNode, address);
       index++;
     }
     
     if (headerEntry.useToggle)
-      headerEntry.enclosingBox.buildViews(gNumAddressesToShow);
+      headerEntry.enclosingBox.buildViews();
   } // if msgheader parser
 }
 
-
-function setFromBuddyIcon(email)
+function updateEmailAddressNode(emailAddressNode, address)
 {
-   var fromBuddyIcon = document.getElementById("fromBuddyIcon");
-
-   try {
-     // better to cache this?
-     var myScreenName = pref.getCharPref("aim.session.screenname");
-
-     if (!abAddressCollector)
-       abAddressCollector = Components.classes[abAddressCollectorContractID].getService(Components.interfaces.nsIAbAddressCollecter);
-
-     var card = abAddressCollector.getCardFromAttribute("PrimaryEmail", email);
-
-     if (myScreenName && card && card.aimScreenName) {
-       if (!gIOService) {
-         // lazily create these globals
-         gIOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
-         gFileHandler = gIOService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
-         
-         var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
-             .getService(Components.interfaces.nsIProperties);
-         var profileDir = dirService.get("ProfD", Components.interfaces.nsIFile);
-         gProfileDirURL = gIOService.newFileURI(profileDir);
-       }
-
-       // if we did have a buddy icon on disk for this screenname, this would be the file url spec for it
-       var iconURLStr = gProfileDirURL.spec + "/NIM/" + myScreenName + "/picture/" + card.aimScreenName + ".gif";
-
-       // check if the file exists
-       // is this a perf hit?  (how expensive is stat()?)
-       var iconFile = gFileHandler.getFileFromURLSpec(iconURLStr);
-       if (iconFile.exists()) {
-         fromBuddyIcon.setAttribute("src", iconURLStr);
-         return;
-       }
-     }
-   }
-   catch (ex) {
-     // can get here if no screenname
-     //dump("ex = " + ex + "\n");
-   }
-   fromBuddyIcon.setAttribute("src", "");
-}
-
-function updateEmailAddressNode(emailAddressNode, emailAddress, fullAddress, displayName, useShortView)
-{
-  if (useShortView && displayName) {
-    emailAddressNode.setAttribute("label", displayName);
-    emailAddressNode.setAttribute("tooltiptext", emailAddress);
-  } else {
-    emailAddressNode.setAttribute("label", fullAddress);
-    emailAddressNode.removeAttribute("tooltiptext");
-  }
-
-  emailAddressNode.setTextAttribute("emailAddress", emailAddress);
-  emailAddressNode.setTextAttribute("fullAddress", fullAddress);  
-  emailAddressNode.setTextAttribute("displayName", displayName);  
+  emailAddressNode.setAttribute("label", address.fullAddress || address.displayName);
+  emailAddressNode.removeAttribute("tooltiptext");
+  emailAddressNode.setAttribute("emailAddress", address.emailAddress);
+  emailAddressNode.setAttribute("fullAddress", address.fullAddress);
+  emailAddressNode.setAttribute("displayName", address.displayName);
   
-  if ("AddExtraAddressProcessing" in this)
-    AddExtraAddressProcessing(emailAddress, emailAddressNode);
+  AddExtraAddressProcessing(address.emailAddress, emailAddressNode);
 }
 
 // thunderbird has smart logic for determining if we should show just the display name.
@@ -960,8 +895,8 @@ function updateEmailAddressNode(emailAddressNode, emailAddress, fullAddress, dis
 
 function AddExtraAddressProcessing(emailAddress, addressNode)
 {
-  var displayName = addressNode.getTextAttribute("displayName");  
-  var emailAddress = addressNode.getTextAttribute("emailAddress");
+  var displayName = addressNode.getAttribute("displayName");  
+  var mailAddress = addressNode.getAttribute("emailAddress");
 
   // always show the address for the from and reply-to fields
   var parentElementId = addressNode.parentNode.id;
@@ -969,10 +904,11 @@ function AddExtraAddressProcessing(emailAddress, addressNode)
   if (parentElementId == "expandedfromBox" || parentElementId == "expandedreply-toBox")
     condenseName = false;
 
-  if (condenseName && gShowCondensedEmailAddresses && displayName && useDisplayNameForAddress(emailAddress))
+  if (condenseName && gShowCondensedEmailAddresses && displayName)
   {
-    addressNode.setAttribute('label', displayName); 
-    addressNode.setAttribute("tooltiptext", emailAddress);
+    if (useDisplayNameForAddress(mailAddress))
+      addressNode.setAttribute("label", displayName);
+    addressNode.setAttribute("tooltiptext", mailAddress);
     addressNode.setAttribute("tooltip", "emailAddressTooltip");
   }
   else
@@ -982,7 +918,7 @@ function AddExtraAddressProcessing(emailAddress, addressNode)
 function fillEmailAddressPopup(emailAddressNode)
 {
   var emailAddressPlaceHolder = document.getElementById('emailAddressPlaceHolder');
-  var emailAddress = emailAddressNode.getTextAttribute('emailAddress');
+  var emailAddress = emailAddressNode.getAttribute('emailAddress');
 
   emailAddressPlaceHolder.setAttribute('label', emailAddress);
 }
@@ -991,7 +927,7 @@ function fillEmailAddressPopup(emailAddressNode)
 // Public method called to generate a tooltip over a condensed display name
 function fillInEmailAddressTooltip(addressNode)
 {
-  var emailAddress = addressNode.getTextAttribute('emailAddress');
+  var emailAddress = addressNode.getAttribute('emailAddress');
   var tooltipNode = document.getElementById("attachmentListTooltip");
   tooltipNode.setAttribute("label", attachmentName);
   return true;
@@ -1141,7 +1077,7 @@ function detachAttachment(aAttachment, aSaveFirst)
 function CanDetachAttachments()
 {
   var uri = GetLoadedMessage();
-  var canDetach = !IsNewsMessage(uri) && (!IsImapMessage(uri) || CheckOnline());
+  var canDetach = !IsNewsMessage(uri) && (!IsImapMessage(uri) || MailOfflineMgr.isOnline());
   if (canDetach && ("content-type" in currentHeaderData))
   {
     var contentType = currentHeaderData["content-type"].headerValue;
@@ -1258,16 +1194,28 @@ function cloneAttachment(aAttachment)
 function displayAttachmentsForExpandedView()
 {
   var numAttachments = currentAttachments.length;
+  var expandedAttachmentBox = document.getElementById('attachmentView');
+  var attachmentSplitter = document.getElementById('attachment-splitter');
   
+  if (numAttachments <= 0)
+  {
+    expandedAttachmentBox.collapsed = true;
+    attachmentSplitter.collapsed = true;
+  }
+  else if (!gBuildAttachmentsForCurrentMsg)
+  {
   // IMPORTANT: make sure we uncollapse the attachment box BEFORE we start adding
   // our attachments to the view. Otherwise, layout doesn't calculate the correct height for
   // the attachment view and we end up with a box that is too tall.
+    expandedAttachmentBox.collapsed = false;
+    attachmentSplitter.collapsed = false;
+    
+    if (gShowLargeAttachmentView)
+      expandedAttachmentBox.setAttribute("largeView", "true");
 
-  var expandedAttachmentBox = document.getElementById('attachmentView');
-  expandedAttachmentBox.collapsed = numAttachments <= 0;
+    // Remove height attribute, or the attachments box could be drawn badly:
+    expandedAttachmentBox.removeAttribute("height");
 
-  if (numAttachments > 0 && !gBuildAttachmentsForCurrentMsg)
-  {
     var attachmentList = document.getElementById('attachmentList');
     for (index in currentAttachments)
     {
@@ -1279,10 +1227,7 @@ function displayAttachmentsForExpandedView()
       attachmentView.setAttribute("class", "descriptionitem-iconic"); 
 
       if (gShowLargeAttachmentView)
-      {
         attachmentView.setAttribute("largeView", "true");
-        attachmentView.setAttribute("orient", "vertical");
-      }
 
       setApplicationIconForAttachment(attachment, attachmentView, gShowLargeAttachmentView);
       attachmentView.setAttribute("tooltip", "attachmentListTooltip");
@@ -1296,6 +1241,18 @@ function displayAttachmentsForExpandedView()
       var item = attachmentList.appendChild(attachmentView);
     } // for each attachment
     gBuildAttachmentsForCurrentMsg = true;
+
+    // Switch overflow off (via css attribute selector) temporarily to get the preferred window height:
+    var attachmentContainer = document.getElementById('attachmentView');
+    attachmentContainer.setAttribute("attachmentOverflow", "false");
+    var attachmentHeight = expandedAttachmentBox.boxObject.height;
+    attachmentContainer.setAttribute("attachmentOverflow", "true");
+
+    // If the attachments box takes up too much of the message pane, downsize:
+    var maxAttachmentHeight = document.getElementById('messagepanebox').boxObject.height / 4;
+    if (attachmentHeight > maxAttachmentHeight)
+      attachmentHeight = maxAttachmentHeight;
+    expandedAttachmentBox.setAttribute("height", attachmentHeight);
   }
 }
 
@@ -1306,7 +1263,7 @@ function setApplicationIconForAttachment(attachment, listitem, largeView)
   var iconSize = largeView ? kLargeIcon : kSmallIcon;
   // generate a moz-icon url for the attachment so we'll show a nice icon next to it.
   if ( attachment.contentType == 'text/x-moz-deleted' )
-    listitem.setAttribute('image', 'chrome://messenger/skin/icons/message-mail-attach-del.gif');
+    listitem.setAttribute('image', 'chrome://messenger/skin/icons/message-mail-attach-del.png');
   else
     listitem.setAttribute('image', "moz-icon:" + "//" + attachment.displayName + "?size=" + iconSize + "&contentType=" + attachment.contentType);
 }
@@ -1441,7 +1398,7 @@ function addAttachmentToPopup(popup, attachment, attachmentIndex)
       }
       var canDetach = !(/news-message:/.test(attachment.uri)) && 
           !signedOrEncrypted &&
-          (!(/imap-message/.test(attachment.uri)) || CheckOnline());
+          (!(/imap-message/.test(attachment.uri)) || MailOfflineMgr.isOnline());
       menuitementry.setAttribute('label', gOpenLabel); 
       menuitementry.setAttribute('accesskey', gOpenLabelAccesskey); 
       menuitementry = openpopup.appendChild(menuitementry);
@@ -1550,9 +1507,19 @@ function ClearAttachmentList()
 
 function ShowEditMessageButton() 
 {
-  var editBox = document.getElementById("editMessageBox");
-  if (editBox)
-    editBox.collapsed = false;
+  // it would be nice if we passed in the msgHdr from the back end
+  var msgHdr;
+  try
+  {
+    msgHdr = gDBView.hdrForFirstSelectedMessage;
+  }
+  catch (ex)
+  {
+    return;
+  }
+
+  if (IsSpecialFolder(msgHdr.folder, MSG_FOLDER_FLAG_DRAFTS, true))
+    document.getElementById("editMessageBox").collapsed = false;
 } 
 
 function ClearEditMessageButton() 
