@@ -1,4 +1,4 @@
-//@line 43 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/base/content/msgMail3PaneWindow.js"
+//@line 43 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/base/content/msgMail3PaneWindow.js"
 
 /* This is where functions related to the 3 pane window are kept */
 
@@ -10,6 +10,9 @@ const kMailCheckOncePrefName = "mail.startup.enabledMailCheckOnce";
 const kStandardPaneConfig = 0;
 const kWidePaneConfig = 1;
 const kVerticalPaneConfig = 2;
+const kWideThreadPaneConfig = 3;
+
+const kNumFolderViews = 4; // total number of folder views
 
 // from nsMsgFolderFlags.h
 const MSG_FOLDER_FLAG_ELIDED = 0x10;
@@ -23,7 +26,7 @@ var gThreadAndMessagePaneSplitter = null;
 var gUnreadCount = null;
 var gTotalCount = null;
 var gCurrentPaneConfig = 0;
-
+var gCurrentFolderView;  
 var gCurrentLoadingFolderURI;
 var gCurrentFolderToReroot;
 var gCurrentLoadingFolderSortType = 0;
@@ -31,7 +34,6 @@ var gCurrentLoadingFolderSortOrder = 0;
 var gCurrentLoadingFolderViewType = 0;
 var gCurrentLoadingFolderViewFlags = 0;
 var gRerootOnFolderLoad = false;
-var gCurrentDisplayedMessage = null;
 var gNextMessageAfterDelete = null;
 var gNextMessageAfterLoad = null;
 var gNextMessageViewIndexAfterDelete = -2;
@@ -83,7 +85,8 @@ function SelectAndScrollToKey(aMsgKey)
 // returns true if we ended up scrolling to a message
 function ScrollToMessageAfterFolderLoad (folder)
 {
-  var scrolled = ScrollToMessage(nsMsgNavigationType.firstNew, true, false /* selectMessage */);
+  var scrolled = pref.getBoolPref("mailnews.scroll_to_new_message") &&
+      ScrollToMessage(nsMsgNavigationType.firstNew, true, false /* selectMessage */);
   if (!scrolled && folder && pref.getBoolPref("mailnews.remember_selected_message")) 
   {
     // if we failed to scroll to a new message,
@@ -128,9 +131,10 @@ var folderListener = {
     OnItemPropertyChanged: function(item, property, oldValue, newValue) { },
 
     OnItemIntPropertyChanged: function(item, property, oldValue, newValue) {
-      if (item == msgWindow.openFolder) {
+      if (item == gMsgFolderSelected) {
         if(property.toString() == "TotalMessages" || property.toString() == "TotalUnreadMessages") {
           UpdateStatusMessageCounts(gMsgFolderSelected);
+          UpdateFolderLocationPicker(item);
         }      
       }
     },
@@ -153,6 +157,7 @@ var folderListener = {
              gCurrentFolderToReroot = null;
              if(msgFolder) {
                msgFolder.endFolderLoading();
+               UpdateStatusQuota(msgFolder);
                // suppress command updating when rerooting the folder
                // when rerooting, we'll be clearing the selection
                // which will cause us to update commands.
@@ -174,8 +179,6 @@ var folderListener = {
                  gDBView.suppressCommandUpdating = false;
                }
 
-               gIsEditableMsgFolder = IsSpecialFolder(msgFolder, MSG_FOLDER_FLAG_DRAFTS, true);
-
                gCurrentLoadingFolderSortType = 0;
                gCurrentLoadingFolderSortOrder = 0;
                gCurrentLoadingFolderViewType = 0;
@@ -232,7 +235,8 @@ var folderListener = {
              {
                 gDefaultSearchViewTerms = null;
                 viewDebug("searching gVirtualFolderTerms\n");
-                loadVirtualFolder(); 
+                gDBView.viewFolder = gMsgFolderSelected;
+                ViewChangeByFolder(gMsgFolderSelected);
              }
              else if (gMsgFolderSelected.flags & MSG_FOLDER_FLAG_VIRTUAL)
              {
@@ -244,18 +248,15 @@ var folderListener = {
                // get the view value from the folder
                if (msgFolder)
                {
-                 var msgDatabase = msgFolder.getMsgDatabase(msgWindow);
-                 var dbFolderInfo = msgDatabase.dBFolderInfo; 
-                 var result = dbFolderInfo.getUint32Property("current-view", 0);
-                 
                  // if our new view is the same as the old view and we already have the list of search terms built up
                  // for the old view, just re-use it
-                 if (gCurrentViewValue == result && gDefaultSearchViewTerms)
+                 var result = GetMailViewForFolder(msgFolder);
+                 if (GetSearchInput() && gCurrentViewValue == result && gDefaultSearchViewTerms)
                  {
                    viewDebug("searching gDefaultSearchViewTerms and rerootingFolder\n");
                    Search("");
                  }
-                 else
+                 else if (document.getElementById("mailviews-container")) // only load the folder view if the views toolbar is visible
                  {
                    viewDebug("changing view by value\n");
                    ViewChangeByValue(result);
@@ -263,7 +264,7 @@ var folderListener = {
                }
              }
            }
-         } 
+         }
        } 
        else if (eventType == "ImapHdrDownloaded") {
          if (folder) {
@@ -292,6 +293,10 @@ var folderListener = {
        else if (eventType == "DeleteOrMoveMsgFailed") {
          HandleDeleteOrMoveMsgFailed(folder);
        }
+       else if (eventType == "AboutToCompact") {
+        if (gDBView)
+          gCurrentlyDisplayedMessage = gDBView.currentlyDisplayedMessage;
+       }
        else if (eventType == "CompactCompleted") {
          HandleCompactCompleted(folder);
        }
@@ -315,8 +320,29 @@ var folderObserver = {
         DropOnFolderTree(row, orientation);
     },
 
-    onToggleOpenState: function()
+    onToggleOpenState: function(index)
     {
+      var folderTree = GetFolderTree();
+
+      // Nothing to do when collapsing an item.
+      if (folderTree.view.isContainerOpen(index))
+        return;
+
+      var folderResource = GetFolderResource(folderTree, index);
+
+      if (folderTree.view.getLevel(index) == 0)
+      {
+        // (Imap/Nntp/Pop) Account item.
+
+        folderResource.QueryInterface(Components.interfaces.nsIMsgFolder)
+                      .server.performExpand(msgWindow);
+      }
+      else if (folderResource instanceof Components.interfaces.nsIMsgImapMailFolder)
+      {
+        // Imap message folder item.
+
+        folderResource.performExpand(msgWindow);
+      }
     },
 
     onCycleHeader: function(colID, elt)
@@ -509,6 +535,7 @@ function HandleCompactCompleted(folder)
           viewType = dbFolderInfo.viewType;
           dbFolderInfo = null;
         }
+        
         RerootFolder(uri, msgFolder, viewType, viewFlags, sortType, sortOrder);
         LoadCurrentlyDisplayedMessage();
       }
@@ -626,21 +653,20 @@ function UpdateMailPaneConfig(aMsgWindowInitialized) {
     document.getElementById('messagepanebox').setAttribute('flex', 1);
   else 
     document.getElementById('messagepanebox').removeAttribute('flex');
-    
-  
+      
   // don't do anything if we are already in the correct configuration
   if (paneConfig == gCurrentPaneConfig)
     return;
 
   var mailContentWrapper = document.getElementById("mailContentWrapper");
   var messagesBox = document.getElementById("messagesBox");
+  var messengerBox = document.getElementById("messengerBox");
   var messagePaneBox = GetMessagePane();
   var msgPaneReRooted = false;
-
   var threadPaneSplitter = GetThreadAndMessagePaneSplitter();
 
   // the only element we need to re-root is the message pane.
-  var desiredMsgPaneParentId = (paneConfig == "0" || paneConfig == "2") ? "messagesBox" : "mailContentWrapper";
+  var desiredMsgPaneParentId = (paneConfig == "0" || paneConfig == "2" || paneConfig == "3") ? "messagesBox" : "mailContentWrapper";
 
   if (messagePaneBox.parentNode.id != desiredMsgPaneParentId)
   {
@@ -653,6 +679,20 @@ function UpdateMailPaneConfig(aMsgWindowInitialized) {
      messagePaneNewParent.appendChild(messagePaneBox); 
      msgPaneReRooted = true;
   }
+  
+  /* this code doesn't work yet, see the comment below about kWideThreadPaneConfig
+  if (gCurrentPaneConfig == kWideThreadPaneConfig)
+  {   
+    threadPaneSplitter.setAttribute("orient", "vertical");
+    mailContentWrapper.setAttribute("orient", "horizontal");
+    mailContentWrapper.removeChild(threadPaneSplitter);
+    mailContentWrapper.removeChild(messagePaneBox);
+    messagesBox.insertBefore(threadPaneSplitter, messagesBox.firstChild);
+    messagesBox.insertBefore(messengerBox, messagesBox.firstChild);
+    messagePaneBox.removeAttribute("flex");
+    msgPaneReRooted = true;
+  }
+  */
 
   // now for each config, handle any extra clean up to create that view (such as changing a box orientation)
   if (paneConfig == kStandardPaneConfig) // standard 3-Pane Layout
@@ -663,7 +703,6 @@ function UpdateMailPaneConfig(aMsgWindowInitialized) {
     mailContentWrapper.setAttribute("orient", "horizontal");
     messagesBox.setAttribute("orient", "vertical");
   }
-
   else if (paneConfig == kWidePaneConfig)  // "Wide" Window Pane Layout
   {     
     threadPaneSplitter.setAttribute("orient", "vertical");
@@ -671,8 +710,7 @@ function UpdateMailPaneConfig(aMsgWindowInitialized) {
     // finally, make sure mailContentWrapper has the correct orientation
     mailContentWrapper.setAttribute("orient", "vertical");
     messagesBox.setAttribute("orient", "vertical");
-  }
-  
+  }  
   else if (paneConfig == kVerticalPaneConfig) // Vertical Pane Layout
   {
     messagesBox.setAttribute("orient", "horizontal");
@@ -680,6 +718,22 @@ function UpdateMailPaneConfig(aMsgWindowInitialized) {
     // finally, make sure mailContentWrapper has the correct orientation
     mailContentWrapper.setAttribute("orient", "horizontal");
   }
+  else if (paneConfig == kWideThreadPaneConfig) 
+  {
+    // kWideThreadPaneConfig is a easter egg layout which isn't fully polished. So 
+    // the menu item for selecting it is hidden from the UI.
+    // If you change from kWideThreadPaneConfig to another layout, you have to restart
+    // Thunderbird before things look right. Loading account central looks really bad.
+    // When you change into this layout, the thread pane gets re-rooted and we don't
+    // handle that properly, the user must re-select the folder before the thread pane
+    // relists the messages in it.
+    mailContentWrapper.insertBefore(threadPaneSplitter, mailContentWrapper.firstChild);
+    mailContentWrapper.insertBefore(messengerBox, mailContentWrapper.firstChild);
+
+    mailContentWrapper.setAttribute("orient", "vertical");
+    threadPaneSplitter.setAttribute("orient", "vertical");
+    messagePaneBox.setAttribute("flex", "1");
+  }
 
   // re-rooting the message pane causes the docshell to get destroyed 
   // and replaced with another one. As such, we need to re-set the window (and thus the 
@@ -693,7 +747,7 @@ function UpdateMailPaneConfig(aMsgWindowInitialized) {
     messenger.SetWindow(window, msgWindow);
     MsgReload(); 
   }
-  
+
   // record the new configuration
   gCurrentPaneConfig = paneConfig; 
 }
@@ -730,8 +784,8 @@ function OnLoadMessenger()
     }
     else // for higher resolution displays, use larger values for height and width
     {
-      defaultWidth = screen.availWidth * .8;
-      defaultHeight = screen.availHeight * .8; 
+      defaultWidth = screen.availWidth <= 1024 ? screen.availWidth * .95 : screen.availWidth * .8;      
+      defaultHeight = screen.availHeight * .8;
     }
 
     document.documentElement.setAttribute("width", defaultWidth);
@@ -749,21 +803,27 @@ function delayedOnLoadMessenger()
   pref.addObserver("mail.pane_config.dynamic", MailPrefObserver, false);
   pref.addObserver("mail.showFolderPaneColumns", MailPrefObserver, false);
 
-  AddMailOfflineObserver();
+  MailOfflineMgr.init();
   CreateMailWindowGlobals();
+  accountCentralBox = document.getElementById("accountCentralBox");
+  GetMessagePane().collapsed = true;
   verifyAccounts(null);
     
-  ShowThreadPane();
   InitMsgWindow();
   messenger.SetWindow(window, msgWindow);
 
   InitializeDataSources();
   InitPanes();
+  MigrateAttachmentDownloadStore();
+  MigrateJunkMailSettings();
+  MigrateFolderViews();
 
   accountManager.setSpecialFolders();
   accountManager.loadVirtualFolders();
   accountManager.addIncomingServerListener(gThreePaneIncomingServerListener);
-
+  
+  gPhishingDetector.init();
+  
   AddToSession();
 
   //need to add to session before trying to load start folder otherwise listeners aren't
@@ -787,6 +847,24 @@ function delayedOnLoadMessenger()
     gSearchEmailAddress = (window.arguments.length > 2) ? window.arguments[2] : null;
   }
 
+//@line 892 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/base/content/msgMail3PaneWindow.js"
+  var nsIShellService = Components.interfaces.nsIShellService;
+  var shellService;
+  var defaultAccount;
+  try {
+    shellService = Components.classes["@mozilla.org/mail/shell-service;1"].getService(nsIShellService);
+    defaultAccount = accountManager.defaultAccount;
+  } catch (ex) {}
+  
+  // show the default client dialog only if we have at least one account, 
+  // if we should check for the default client, 
+  // and we aren't already the default for all of our recognized types (mail, news, rss)
+  if (shellService && defaultAccount && shellService.shouldCheckDefaultClient 
+      && !shellService.isDefaultClient(true, nsIShellService.MAIL))
+    window.openDialog("chrome://messenger/content/defaultClientDialog.xul", "DefaultClient", 
+                      "modal,centerscreen,chrome,resizable=no");
+//@line 908 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/base/content/msgMail3PaneWindow.js"
+
   setTimeout("loadStartFolder(gStartFolderUri);", 0);
 
   // FIX ME - later we will be able to use onload from the overlay
@@ -798,7 +876,9 @@ function delayedOnLoadMessenger()
 
   //Set focus to the Thread Pane the first time the window is opened.
   SetFocusThreadPane();
-
+  
+  gFindBar.initFindBar();
+  
   // initialize the customizeDone method on the customizeable toolbar
   var toolbox = document.getElementById("mail-toolbox");
   toolbox.customizeDone = MailToolboxCustomizeDone;
@@ -814,10 +894,14 @@ function OnUnloadMessenger()
   pref.QueryInterface(Components.interfaces.nsIPrefBranch2);
   pref.removeObserver("mail.pane_config.dynamic", MailPrefObserver);
   pref.removeObserver("mail.showFolderPaneColumns", MailPrefObserver);
+  
+  gFindBar.uninitFindBar();
 
   // FIX ME - later we will be able to use onload from the overlay
   OnUnloadMsgHeaderPane();
 
+  OnUnloadFolderPane();
+
   OnMailWindowUnload();
 }
 
@@ -883,12 +967,6 @@ function loadStartFolder(initialUri)
 
                 startFolderResource = inboxFolder.QueryInterface(Components.interfaces.nsIRDFResource);
             }
-            else
-            {
-                // set the startFolderResource to the server, so we select it
-                // so we'll get account central
-                startFolderResource = RDF.GetResource(defaultServer.serverURI);
-            }
         }
 
         var startFolder = startFolderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
@@ -901,12 +979,11 @@ function loadStartFolder(initialUri)
         // or a pop3 account that is deferred or deferred to,
         // or the case where initialUri is non-null (non-startup)
         if (!initialUri && isLoginAtStartUpEnabled && gLoadStartFolder
-            && !defaultServer.isDeferredTo &&
+            && !defaultServer.isDeferredTo && !gNewAccountToLoad &&
             defaultServer.rootFolder == defaultServer.rootMsgFolder)
           defaultServer.PerformBiff(msgWindow);        
 
-        SelectFolder(startFolder.URI);        
-
+        SelectFolder(startFolder.URI);
 
         // because the "open" state persists, we'll call
         // PerformExpand() for all servers that are open at startup.
@@ -918,8 +995,8 @@ function loadStartFolder(initialUri)
     }
     catch(ex)
     {
-
-      if (initialUri)
+      // this is the case where we're trying to auto-subscribe to a folder.
+      if (initialUri && !startFolder.parent)
       {
         messenger.loadURL(window, initialUri);
         return;
@@ -928,55 +1005,35 @@ function loadStartFolder(initialUri)
       dump(ex);
       dump('Exception in LoadStartFolder caused by no default account.  We know about this\n');
     }
-    
+
     // if gLoadStartFolder is true, then we must have just created a POP3 account
     // and we aren't supposed to initially download mail. (Bug #270743)
     if (gLoadStartFolder)
       MsgGetMessagesForAllServers(defaultServer);
 
-    if (this.CheckForUnsentMessages != undefined && CheckForUnsentMessages())
-    {
-      var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
-      if (!ioService.offline) 
-      {
-        InitPrompts();
-        InitServices();
-
-        if (gPromptService) 
-        {
-          var buttonPressed = gPromptService.confirmEx(window, 
-                            gOfflinePromptsBundle.getString('sendMessagesOfflineWindowTitle'), 
-                            gOfflinePromptsBundle.getString('sendMessagesLabel'),
-                            gPromptService.BUTTON_TITLE_IS_STRING * (gPromptService.BUTTON_POS_0 + 
-                            gPromptService.BUTTON_POS_1),
-                            gOfflinePromptsBundle.getString('sendMessagesSendButtonLabel'),
-                            gOfflinePromptsBundle.getString('sendMessagesNoSendButtonLabel'),
-                            null, null, {value:0});
-          if (buttonPressed == 0) 
-            SendUnsentMessages();
-        }
-      }
-    }
+    // if appropriate, send unsent messages. This may end up prompting the user
+    if (MailOfflineMgr.isOnline() && MailOfflineMgr.shouldSendUnsentMessages())
+      SendUnsentMessages();
 }
 
 function AddToSession()
 {
-    try {
-        var mailSession = Components.classes[mailSessionContractID].getService(Components.interfaces.nsIMsgMailSession);
-        
-        var nsIFolderListener = Components.interfaces.nsIFolderListener;
-        var notifyFlags = nsIFolderListener.intPropertyChanged | nsIFolderListener.event;
-        mailSession.AddFolderListener(folderListener, notifyFlags);
-	} catch (ex) {
-        dump("Error adding to session\n");
-    }
+  try {
+    var mailSession = Components.classes[mailSessionContractID]
+                                .getService(Components.interfaces.nsIMsgMailSession);
+    var nsIFolderListener = Components.interfaces.nsIFolderListener;
+    var notifyFlags = nsIFolderListener.intPropertyChanged | nsIFolderListener.event;
+    mailSession.AddFolderListener(folderListener, notifyFlags);
+  } catch (ex) {
+    dump("Error adding to session\n");
+  }
 }
 
 function InitPanes()
 {
-    OnLoadFolderPane();
-    OnLoadThreadPane();
-    SetupCommandUpdateHandlers();
+  OnLoadFolderPane();
+  OnLoadThreadPane();
+  SetupCommandUpdateHandlers();
 }
 
 function InitializeDataSources()
@@ -993,67 +1050,139 @@ function InitializeDataSources()
 
 function OnFolderUnreadColAttrModified(event)
 {
-    if (event.attrName == "hidden")
-    {
-        var folderNameCell = document.getElementById("folderNameCell");
-        var label = {"true": "?folderTreeName", "false": "?folderTreeSimpleName"};
-        folderNameCell.setAttribute("label", label[event.newValue]);
-    }
+  if (event.attrName == "hidden")
+  {
+    var folderNameCell = document.getElementById("folderNameCell");
+    var label = {"true": "?folderTreeName", "false": "?folderTreeSimpleName"};
+    folderNameCell.setAttribute("label", label[event.newValue]);
+  }
 }
 
 function UpdateFolderColumnVisibility()
 {
-    var folderNameCol = document.getElementById("folderNameCol");
-    var showColumns = pref.getBoolPref("mail.showFolderPaneColumns");
-    var folderUnreadCol = document.getElementById("folderUnreadCol"); 
-    var folderColumnLabel = document.getElementById("folderColumnLabel");
-    if (!showColumns)
-    {
-      var folderTotalCol = document.getElementById("folderTotalCol");
-      var folderSizeCol = document.getElementById("folderSizeCol");
-      folderUnreadCol.setAttribute("hidden", "true");
-      folderTotalCol.setAttribute("hidden", "true");
-      folderSizeCol.setAttribute("hidden", "true");
-      folderNameCol.removeAttribute("label");
-    }
-    else
-    {
-      folderNameCol.setAttribute("label", folderColumnLabel.value);
-    }
+  var folderNameCol = document.getElementById("folderNameCol");
+  var showColumns = pref.getBoolPref("mail.showFolderPaneColumns");
+  var folderUnreadCol = document.getElementById("folderUnreadCol"); 
+  var folderColumnLabel = document.getElementById("folderColumnLabel");
+  if (!showColumns)
+  {
+    var folderTotalCol = document.getElementById("folderTotalCol");
+    var folderSizeCol = document.getElementById("folderSizeCol");
+    folderUnreadCol.setAttribute("hidden", "true");
+    folderTotalCol.setAttribute("hidden", "true");
+    folderSizeCol.setAttribute("hidden", "true");
+    folderNameCol.removeAttribute("label");
+  }
+  else
+  {
+    folderNameCol.setAttribute("label", folderColumnLabel.value);
+  }
 
-    folderNameCol.setAttribute("hideheader", showColumns ? "false" : "true");
-    var folderPaneHeader = document.getElementById("folderPaneHeader");
-    folderPaneHeader.setAttribute("hidden", showColumns ? "true" : "false");
-    var folderTree = document.getElementById("folderTree");
-    folderTree.setAttribute("hidecolumnpicker", showColumns ? "false" : "true");
-    var hidden = folderUnreadCol.getAttribute("hidden");
-    if (hidden != "true")
-    {
-        var folderNameCell = document.getElementById("folderNameCell");
-        folderNameCell.setAttribute("label", "?folderTreeSimpleName");
-    }
+  folderNameCol.setAttribute("hideheader", showColumns ? "false" : "true");
+  var folderTree = document.getElementById("folderTree");
+  folderTree.setAttribute("hidecolumnpicker", showColumns ? "false" : "true");
+  var hidden = folderUnreadCol.getAttribute("hidden");
+  if (hidden != "true")
+  {
+    var folderNameCell = document.getElementById("folderNameCell");
+    folderNameCell.setAttribute("label", "?folderTreeSimpleName");
+  }
 } 
 
+// loadFolderViewForTree -- a helper routine split away from
+// loadFolderView.
+// returns a localized string corresponding to the name of the new view
+function loadFolderViewForTree(aNewFolderView, aFolderTree)
+{
+  var folderPaneHeader = document.getElementById('folderpane-title');
+  var database = aFolderTree.database;
+  var nsIRDFDataSource = Components.interfaces.nsIRDFDataSource;
+
+  // Each folder pane view has the following properties: 
+  // ref-> the ref attribute for the folderTree
+  // label -> the UI label associated with the folder view
+  // datasources -> array of the data sources associated with the view
+
+  var folderViews = [ 
+                      {ref:"msgaccounts:/",           label:"folderPaneHeader",           dataSources: [accountManagerDataSource.QueryInterface(nsIRDFDataSource),
+                                                                                                        folderDataSource.QueryInterface(nsIRDFDataSource)] },
+                      {ref:"mailnewsunreadfolders:/", label:"folderPaneHeader_unread",    dataSources: [unreadFolderDataSource.QueryInterface(nsIRDFDataSource)]},
+                      {ref:"mailnewsfavefolders:/",   label:"folderPaneHeader_favorites", dataSources: [favoriteFoldersDataSource.QueryInterface(nsIRDFDataSource)]},
+                      {ref:"mailnewsrecentfolders:/", label:"folderPaneHeader_recent",    dataSources: [recentFoldersDataSource.QueryInterface(nsIRDFDataSource)]},
+                    ];
+
+  // unload the current data sources
+  if (gCurrentFolderView != undefined)
+  {
+    var dataSourcesToUnload = folderViews[gCurrentFolderView].dataSources;
+    for (index in dataSourcesToUnload)
+      database.RemoveDataSource(dataSourcesToUnload[index]);
+  }
+
+  // add the new data sources
+  var dataSourcesToAdd = folderViews[aNewFolderView].dataSources;
+  for (index in dataSourcesToAdd)
+  {
+    database.AddDataSource(dataSourcesToAdd[index]);
+    var msgDS = dataSourcesToAdd[index].QueryInterface(Components.interfaces.nsIMsgRDFDataSource);
+    msgDS.window = msgWindow;
+  }
+
+  aFolderTree.setAttribute('ref', folderViews[aNewFolderView].ref);
+  return gMessengerBundle.getString(folderViews[aNewFolderView].label);
+}
+
+function loadFolderView(aNewFolderView)
+{
+  if (gCurrentFolderView && (gCurrentFolderView == aNewFolderView))
+    return;
+    
+  var folderTree = GetFolderTree();
+
+  var folderPaneHeader = document.getElementById('folderpane-title');
+  var folderTree = GetFolderTree();
+  var database = GetFolderDatasource();
+  
+  // load the folder view into the folder pane
+  folderPaneHeader.value = loadFolderViewForTree(aNewFolderView, GetFolderTree());
+  
+  // if the folder location picker is visible, load the folder view into the location
+  // picker as well. 
+  var folderLocationPicker = document.getElementById('folder-location-container');
+  if (folderLocationPicker)
+    loadFolderViewForTree(aNewFolderView, document.getElementById('folderLocationPopup').tree);
+
+  // now reflect the new value back into prefs
+  pref.setIntPref('mail.ui.folderpane.view', gCurrentFolderView = aNewFolderView);
+}
+
+// we can cycle the folder view forward or backwards
+function CycleFolderView(aCycleForward)
+{
+  // pass the call onto loadFolderView...
+  var offset = aCycleForward ? 1 : kNumFolderViews - 1;
+  loadFolderView((gCurrentFolderView + offset) % kNumFolderViews);
+}
+
 function OnLoadFolderPane()
 {
-    UpdateFolderColumnVisibility();
-    var folderUnreadCol = document.getElementById("folderUnreadCol");
-    folderUnreadCol.addEventListener("DOMAttrModified", OnFolderUnreadColAttrModified, false);
+  UpdateFolderColumnVisibility();
+  var folderUnreadCol = document.getElementById("folderUnreadCol");
+  folderUnreadCol.addEventListener("DOMAttrModified", OnFolderUnreadColAttrModified, false);
 
-    //Add folderDataSource and accountManagerDataSource to folderPane
-    accountManagerDataSource = accountManagerDataSource.QueryInterface(Components.interfaces.nsIRDFDataSource);
-    folderDataSource = folderDataSource.QueryInterface(Components.interfaces.nsIRDFDataSource);
-    var database = GetFolderDatasource();
+  loadFolderView(pref.getIntPref('mail.ui.folderpane.view'));
 
-    database.AddDataSource(accountManagerDataSource);
-    database.AddDataSource(folderDataSource);
-    var folderTree = GetFolderTree();
-    folderTree.setAttribute("ref", "msgaccounts:/");
+  var folderTree = GetFolderTree();
+  var folderTreeBuilder = folderTree.builder.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
+  folderTreeBuilder.addObserver(folderObserver);
+  folderTree.addEventListener("click",FolderPaneOnClick,true);
+  folderTree.addEventListener("mousedown",TreeOnMouseDown,true);
+}
 
-    var folderTreeBuilder = folderTree.builder.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
-    folderTreeBuilder.addObserver(folderObserver);
-    folderTree.addEventListener("click",FolderPaneOnClick,true);
-    folderTree.addEventListener("mousedown",TreeOnMouseDown,true);
+function OnUnloadFolderPane()
+{
+  var folderTreeBuilder = GetFolderTree().builder.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
+  folderTreeBuilder.removeObserver(folderObserver);
 }
 
 // builds prior to 12-08-2001 did not have the labels column
@@ -1064,7 +1193,6 @@ function OnLoadFolderPane()
 // "mailnews.ui.threadpane.version" pref.
 function UpgradeThreadPaneUI()
 {
-  var labelCol;
   var threadPaneUIVersion;
 
   try {
@@ -1110,40 +1238,80 @@ function UpgradeThreadPaneUI()
 
 function OnLoadThreadPane()
 {
-    UpgradeThreadPaneUI();
+  UpgradeThreadPaneUI();
+}
+
+// folderLocationPickerOnLoad can be called multiple times
+// and it can be called when the location picker isn't in the toolbar
+function folderLocationPickerOnLoad()
+{
+  var folderLocationPicker = document.getElementById('folder-location-container');
+  if (!folderLocationPicker) 
+    return;
+  
+  var locationTree = document.getElementById('folderLocationPopup').tree;
+  locationTree.database.AddDataSource(accountManagerDataSource);
+  locationTree.database.AddDataSource(folderDataSource);
+  locationTree.setAttribute("ref", "msgaccounts:/");
+}
+
+function OnLocationTreeSelect(menulist)
+{
+  SelectFolder(menulist.getAttribute('uri'));
+}
+
+function UpdateFolderLocationPicker(resource)
+{
+  var folderLocationPicker = document.getElementById('folder-location-container');
+  if (!folderLocationPicker) 
+    return;
+  
+  var tree = GetFolderTree();
+  var folders = document.getElementById('locationFolders');
+  var properties = ['BiffState', 'NewMessages', 'HasUnreadMessages',
+                    'SpecialFolder', 'IsServer', 'IsSecure', 'ServerType', 'NoSelect'];
+  var label = GetFolderAttribute(tree, resource, 'FolderTreeName');
+  folders.setAttribute("label", label);
+  for (var i in properties) 
+  {
+    var property = properties[i];
+    var value = GetFolderAttribute(tree, resource, property);
+    folders.setAttribute(property, value);
+  }
+  folders.setAttribute('uri', resource.Value);
 }
 
 function GetFolderDatasource()
 {
-    var folderTree = GetFolderTree();
-    return folderTree.database;
+  var folderTree = GetFolderTree();
+  return folderTree.database;
 }
 
 /* Functions for accessing particular parts of the window*/
 function GetFolderTree()
 {
-    if (! gFolderTree)
-        gFolderTree = document.getElementById("folderTree");
-    return gFolderTree;
+  if (! gFolderTree)
+    gFolderTree = document.getElementById("folderTree");
+  return gFolderTree;
 }
 
 function GetSearchInput()
 {
-    if (gSearchInput) return gSearchInput;
+  if (!gSearchInput)
     gSearchInput = document.getElementById("searchInput");
-    return gSearchInput;
+  return gSearchInput;
 }
 
 function GetMessagePane()
 {
-    if (gMessagePane) return gMessagePane;
+  if (!gMessagePane) 
     gMessagePane = document.getElementById("messagepanebox");
-    return gMessagePane;
+  return gMessagePane;
 }
 
 function GetMessagePaneFrame()
 {
-    return window.content;
+  return window.content;
 }
 
 function FindInSidebar(currentWindow, id)
@@ -1220,10 +1388,9 @@ function ClearMessagePane()
   if(gHaveLoadedMessage)
   {
     gHaveLoadedMessage = false;
-    gCurrentDisplayedMessage = null;
     if (GetMessagePaneFrame().location.href != "about:blank")
         GetMessagePaneFrame().location.href = "about:blank";
-    
+
     // hide the message header view AND the message pane...
     HideMessageHeaderPane();
     gMessageNotificationBar.clearMsgNotifications();
@@ -1264,7 +1431,7 @@ function ChangeSelectionWithoutContentLoad(event, tree)
         if(tree.id == "threadTree")
           gThreadPaneCurrentSelectedIndex = row;
     }
-    event.preventBubble();
+    event.stopPropagation();
 }
 
 function TreeOnMouseDown(event)
@@ -1293,62 +1460,26 @@ function FolderPaneOnClick(event)
     var elt = {};
     folderTree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, elt);
     if (row.value == -1) {
-      if (event.originalTarget.localName == "treecol")
+      if (event.originalTarget.localName == "treecol") {
         // clicking on the name column in the folder pane should not sort
-        event.preventBubble();
-      return;
-    }
-
-    if (elt.value == "twisty")
-    {
-        if (!(folderTree.treeBoxObject.view.isContainerOpen(row.value)))
-        {
-            var folderResource = GetFolderResource(folderTree, row.value);
-            var isServer = GetFolderAttribute(folderTree, folderResource, "IsServer");
-            if (isServer == "true")
-            {
-                var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
-                var server = msgFolder.server;
-                server.performExpand(msgWindow);
-            }
-            else
-            {
-                var serverType = GetFolderAttribute(folderTree, folderResource, "ServerType");
-                if (serverType == "imap")
-                {
-                    var imapFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgImapMailFolder);
-                    imapFolder.performExpand(msgWindow);
-                }
-            }
-        }
+        event.stopPropagation();
+      }
     }
     else if ((event.originalTarget.localName == "slider") ||
              (event.originalTarget.localName == "scrollbarbutton")) {
-      event.preventBubble();
+      event.stopPropagation();
     }
-    else if (event.detail == 2) {
+    else if ((event.detail == 2) && (elt.value != "twisty") &&
+             (folderTree.view.getLevel(row.value) != 0)) {
       FolderPaneDoubleClick(row.value, event);
     }
-
 }
 
 function FolderPaneDoubleClick(folderIndex, event)
 {
-    var folderTree = GetFolderTree();
-    var folderResource = GetFolderResource(folderTree, folderIndex);
-
-    var isServer = GetFolderAttribute(folderTree, folderResource, "IsServer");
-    if (isServer == "true")
-    {
-      if (!(folderTree.treeBoxObject.view.isContainerOpen(folderIndex)))
-      {
-        var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
-        var server = msgFolder.server;
-        server.performExpand(msgWindow);
-      }
-    }
-    else 
+    if (!pref.getBoolPref("mailnews.reuse_thread_window2"))
     {
+      var folderResource = GetFolderResource(GetFolderTree(), folderIndex);
       // Open a new msg window only if we are double clicking on 
       // folders or newsgroups.
       MsgOpenNewWindowForFolder(folderResource.Value, -1 /* key */);
@@ -1356,7 +1487,7 @@ function FolderPaneDoubleClick(folderIndex, event)
       // double clicking should not toggle the open / close state of the
       // folder.  this will happen if we don't prevent the event from
       // bubbling to the default handler in tree.xml
-      event.preventBubble();
+      event.stopPropagation();
     }
 }
 
@@ -1548,15 +1679,16 @@ function EnsureFolderIndex(builder, msgFolder)
 
 function SelectFolder(folderUri)
 {
-    var folderTree = GetFolderTree();
-    var folderResource = RDF.GetResource(folderUri);
-    var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
+  dump("selecting folder " + folderUri + "\n");
+  var folderTree = GetFolderTree();
+  var folderResource = RDF.GetResource(folderUri);
+  var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
 
-    // before we can select a folder, we need to make sure it is "visible"
-    // in the tree.  to do that, we need to ensure that all its
-    // ancestors are expanded
-    var folderIndex = EnsureFolderIndex(folderTree.builderView, msgFolder);
-    ChangeSelection(folderTree, folderIndex);
+  // before we can select a folder, we need to make sure it is "visible"
+  // in the tree.  to do that, we need to ensure that all its
+  // ancestors are expanded
+  var folderIndex = EnsureFolderIndex(folderTree.builderView, msgFolder);
+  ChangeSelection(folderTree, folderIndex);
 }
 
 function SelectMessage(messageUri)
@@ -1573,25 +1705,115 @@ function ReloadMessage()
 
 function GetDBView()
 {
-    return gDBView;
+  return gDBView;
 }
 
 function GetFolderResource(tree, index)
 {
-    return tree.builderView.getResourceAtIndex(index);
+  return tree.builderView.getResourceAtIndex(index);
 }
 
 function GetFolderIndex(tree, resource)
 {
-    return tree.builderView.getIndexOfResource(resource);
+  return tree.builderView.getIndexOfResource(resource);
 }
 
 function GetFolderAttribute(tree, source, attribute)
 {
-    var property = RDF.GetResource("http://home.netscape.com/NC-rdf#" + attribute);
-    var target = tree.database.GetTarget(source, property, true);
-    if (target)
-        target = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
-    return target;
+  var property = RDF.GetResource("http://home.netscape.com/NC-rdf#" + attribute);
+  var target = tree.database.GetTarget(source, property, true);
+  if (target)
+    target = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
+  return target;
+}
+
+function LoadNavigatedToMessage(msgHdr, folder, folderUri)
+{
+  if (IsCurrentLoadedFolder(folder))
+  {
+    gDBView.selectMsgByKey(msgHdr.messageKey);
+  }
+  else
+  {
+    gStartMsgKey = msgHdr.messageKey;
+    SelectFolder(folderUri);
+  }
+  
+}
+
+// Some of the per account junk mail settings have been
+// converted to global prefs. Let's try to migrate some
+// of those settings from the default account.
+function MigrateJunkMailSettings()
+{
+  var junkMailSettingsVersion = pref.getIntPref("mail.spam.version");
+  if (!junkMailSettingsVersion)
+  {
+    // get the default account, check to see if we have values for our 
+    // globally migrated prefs.
+    var defaultAccount;
+    try {
+      defaultAccount = accountManager.defaultAccount;
+    } catch (ex) {}
+    if (defaultAccount && defaultAccount.incomingServer)
+    {
+      // we only care about
+      var prefix = "mail.server." + defaultAccount.incomingServer.key + ".";
+      if (pref.prefHasUserValue(prefix + "manualMark"))
+        pref.setBoolPref("mail.spam.manualMark", pref.getBoolPref(prefix + "manualMark"));
+      if (pref.prefHasUserValue(prefix + "manualMarkMode"))
+        pref.setIntPref("mail.spam.manualMarkMode", pref.getIntPref(prefix + "manualMarkMode"));
+      if (pref.prefHasUserValue(prefix + "spamLoggingEnabled"))
+        pref.setBoolPref("mail.spam.logging.enabled", pref.getBoolPref(prefix + "spamLoggingEnabled"));
+      if (pref.prefHasUserValue(prefix + "markAsReadOnSpam"))
+        pref.setBoolPref("mail.spam.markAsReadOnSpam", pref.getBoolPref(prefix + "markAsReadOnSpam"));
+    }  
+    // bump the version so we don't bother doing this again.
+    pref.setIntPref("mail.spam.version", 1);
+  }
+}
+
+// The first time a user runs a build that supports folder views, pre-populate the favorite folders list
+// with the existing INBOX folders.
+function MigrateFolderViews()
+{
+  var folderViewsVersion = pref.getIntPref("mail.folder.views.version");
+  if (!folderViewsVersion)
+  {
+     var servers = accountManager.allServers;
+     var server;
+     var inbox;
+     for (var index = 0; index < servers.Count(); index++)
+     {
+       server = servers.QueryElementAt(index, Components.interfaces.nsIMsgIncomingServer);
+       if (server)
+       {
+         inbox = GetInboxFolder(server);
+         if (inbox)
+           inbox.setFlag(MSG_FOLDER_FLAG_FAVORITE);
+       }
+     }
+    pref.setIntPref("mail.folder.views.version", 1);
+  }
+}
+
+// Thunderbird has been storing old attachment download meta data in downloads.rdf 
+// even though there was no way to show or clean up this data. Now that we are using 
+// the new download manager in toolkit, we don't want to present this old data.
+// To migrate to the new download manager, remove downloads.rdf.
+function MigrateAttachmentDownloadStore()
+{
+  var attachmentStoreVersion = pref.getIntPref("mail.attachment.store.version");
+  if (!attachmentStoreVersion)
+  {
+    var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
+                     .getService(Components.interfaces.nsIProperties);
+    var downloadsFile = dirService.get("DLoads", Components.interfaces.nsIFile);
+    if (downloadsFile && downloadsFile.exists())
+      downloadsFile.remove(false);
+    
+    // bump the version so we don't bother doing this again.
+    pref.setIntPref("mail.attachment.store.version", 1); 
+  }
 }
 
