@@ -1,4 +1,4 @@
-//@line 36 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/components/preferences/compose.js"
+//@line 38 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/components/preferences/compose.js"
 
 const kLDAPPrefContractID="@mozilla.org/ldapprefs-service;1";
 
@@ -9,6 +9,7 @@ var gComposePane = {
   mDirectories: null,
   mLDAPPrefsService: null,
   mSpellChecker: null,
+  mDictCount : 0,
 
   init: function ()
   {
@@ -16,16 +17,18 @@ var gComposePane = {
       this.mLDAPPrefsService = Components.classes[kLDAPPrefContractID].getService(Components.interfaces.nsILDAPPrefsService);
 
     this.createDirectoriesList();
-    
+
     // build the local address book menu list. We do this by hand instead of using the xul template
-    // builder because of Bug #285076, 
+    // builder because of Bug #285076,
     this.createLocalDirectoriesList();
-    
+
     this.enableAutocomplete();
 
     this.initLanguageMenu();
 
-    document.getElementById('downloadDictionaries').setAttribute('href', xlateURL('urn:clienturl:composer:spellcheckers'));
+    this.populateFonts();
+
+    document.getElementById('downloadDictionaries').setAttribute('href', this.getDictionaryURL());
 
     var preference = document.getElementById("mail.preferences.compose.selectedTabIndex");
     if (preference.value)
@@ -33,6 +36,14 @@ var gComposePane = {
     this.mInitialized = true;
   },
 
+  getDictionaryURL: function()
+  {
+    var formatter = Components.classes["@mozilla.org/toolkit/URLFormatterService;1"]
+                    .getService(Components.interfaces.nsIURLFormatter);
+
+    return formatter.formatURLPref("spellchecker.dictionaries.download.url");
+  },
+
   tabSelectionChanged: function ()
   {
     if (this.mInitialized)
@@ -42,35 +53,29 @@ var gComposePane = {
     }
   },
 
-  showReturnReceipts: function()
-  {
-    document.documentElement.openSubDialog("chrome://messenger/content/preferences/receipts.xul",
-                                           "", null);
-  },
-
   sendOptionsDialog: function()
   {
-    document.documentElement.openSubDialog("chrome://messenger/content/preferences/sendoptions.xul","", null);    
+    document.documentElement.openSubDialog("chrome://messenger/content/preferences/sendoptions.xul","", null);
   },
 
   htmlComposeDialog: function()
   {
-    document.documentElement.openSubDialog("chrome://messenger/content/preferences/htmlcompose.xul","", null);  
+    document.documentElement.openSubDialog("chrome://messenger/content/preferences/htmlcompose.xul","", null);
   },
 
-  enableAutocomplete: function() 
+  enableAutocomplete: function()
   {
-    var directoriesList =  document.getElementById("directoriesList"); 
+    var directoriesList =  document.getElementById("directoriesList");
     var directoriesListPopup = document.getElementById("directoriesListPopup");
     var editButton = document.getElementById("editButton");
 
-    if (document.getElementById("autocompleteLDAP").checked) 
+    if (document.getElementById("autocompleteLDAP").checked)
     {
       editButton.removeAttribute("disabled");
       directoriesList.removeAttribute("disabled");
       directoriesListPopup.removeAttribute("disabled");
     }
-    else 
+    else
     {
       directoriesList.setAttribute("disabled", true);
       directoriesListPopup.setAttribute("disabled", true);
@@ -79,14 +84,14 @@ var gComposePane = {
 
     // if we do not have any directories disable the dropdown list box
     if (!this.mDirectories || (this.mDirectories < 1))
-      directoriesList.setAttribute("disabled", true);  
+      directoriesList.setAttribute("disabled", true);
   },
 
-  createLocalDirectoriesList: function () 
+  createLocalDirectoriesList: function ()
   {
     var abPopup = document.getElementById("abPopup-menupopup");
 
-    if (abPopup) 
+    if (abPopup)
       this.loadLocalDirectories(abPopup);
   },
 
@@ -110,7 +115,7 @@ var gComposePane = {
           item = document.createElement("menuitem");
           item.setAttribute("label", addrbook.dirName);
           item.setAttribute("value", abURI);
-          aPopup.appendChild(item);   
+          aPopup.appendChild(item);
           if (preference.value == abURI)
           {
             aPopup.parentNode.value = abURI;
@@ -125,7 +130,7 @@ var gComposePane = {
   {
     var directoriesListPopup = document.getElementById("directoriesListPopup");
 
-    if (directoriesListPopup) 
+    if (directoriesListPopup)
       this.loadDirectories(directoriesListPopup);
   },
 
@@ -138,30 +143,29 @@ var gComposePane = {
     var arrayOfDirectories;
     var position;
     var dirType;
-    var directoriesList;
     var prefService;
-    
+
     prefService = Components.classes["@mozilla.org/preferences-service;1"]
                             .getService(Components.interfaces.nsIPrefBranch);
-    
-    if (!this.mDirectories) 
+
+    if (!this.mDirectories)
     {
-      try 
+      try
       {
         if (this.mLDAPPrefsService)
           arrayOfDirectories = this.mLDAPPrefsService.getServerList(prefService, prefCount);
       }
       catch (ex) {}
 
-      if (arrayOfDirectories) 
+      if (arrayOfDirectories)
       {
         this.mDirectories = new Array();
         for (var i = 0; i < prefCount.value; i++)
         {
-          if ((arrayOfDirectories[i] != "ldap_2.servers.pab") && 
-            (arrayOfDirectories[i] != "ldap_2.servers.history")) 
+          if ((arrayOfDirectories[i] != "ldap_2.servers.pab") &&
+            (arrayOfDirectories[i] != "ldap_2.servers.history"))
           {
-            try 
+            try
             {
               position = prefService.getIntPref(arrayOfDirectories[i]+".position");
             }
@@ -169,7 +173,7 @@ var gComposePane = {
             {
               position = 1;
             }
-          
+
             try
             {
               dirType = prefService.getIntPref(arrayOfDirectories[i]+".dirType");
@@ -179,7 +183,7 @@ var gComposePane = {
               dirType = 1;
             }
 
-            if ((position != 0) && (dirType == 1)) 
+            if ((position != 0) && (dirType == 1))
             {
               try
               {
@@ -190,59 +194,47 @@ var gComposePane = {
               {
                 description="";
               }
-              
-              if (description != "") 
+
+              if (description != "")
               {
-                if (aPopup) 
+                if (aPopup)
                 {
                   item = document.createElement("menuitem");
                   item.setAttribute("label", description);
                   item.setAttribute("value", arrayOfDirectories[i]);
                   aPopup.appendChild(item);
                 }
-              
+
                 this.mDirectories[j++] = {value:arrayOfDirectories[i], label:description};
               }
             }
           }
         }
-      
-        var value;
-        if (aPopup) 
+
+        if (aPopup)
         {
-          directoriesList =  document.getElementById("directoriesList");
-          if (gRefresh) 
+          // we are in mail/news Account settings
+          item = document.createElement("menuitem");
+          var addressBookBundle = document.getElementById("bundle_addressBook");
+          var directoryName = addressBookBundle.getString("directoriesListItemNone");
+          item.setAttribute("label", directoryName);
+          item.setAttribute("value", "");
+          aPopup.appendChild(item);
+
+          // Now check what we are displaying is valid.
+          var directoriesList =  document.getElementById("directoriesList");
+          var value = directoriesList.value;
+          directoriesList.selectedItem = null;
+          directoriesList.value = value;
+          if (!directoriesList.selectedItem)
           {
-            // gRefresh is true if user edits, removes or adds a directory.
-            value = directoriesList.value;
-            directoriesList.selectedItem = null;
-            directoriesList.value = value;
-            if (!directoriesList.selectedItem)
-              directoriesList.selectedIndex = 0;
-            if (!directoriesList.selectedItem) 
-            {
-              directoriesList.value = "";
+            directoriesList.value = "";
+            // If we have no other directories, also disable the popup.
+            if (gAvailDirectories.length == 0)
               directoriesList.disabled = true;
-            }
-            else if (!prefService.prefIsLocked("ldap_2.autoComplete.directoryServer"))
-              directoriesList.disabled = false;
-            return;
-          }
-        
-          var pref_string_title = "ldap_2.autoComplete.directoryServer";
-          try 
-          {
-            var directoryServer = prefService.getCharPref(pref_string_title);
-            directoriesList.value = directoryServer;
-            if (!directoriesList.selectedItem)
-              directoriesList.selectedIndex = 0;
-            if (!directoriesList.selectedItem)
-              directoriesList.value = "";
-          }
-          catch (ex)
-          {
-            directoriesList.selectedItem = null;
           }
+          else if (!prefService.prefIsLocked("ldap_2.autoComplete.directoryServer"))
+            directoriesList.disabled = false;
         }
       }
     }
@@ -256,7 +248,7 @@ var gComposePane = {
     if (gRefresh)
     {
       var popup = document.getElementById("directoriesListPopup");
-      if (popup) 
+      if (popup)
         while (popup.hasChildNodes())
           popup.removeChild(popup.lastChild);
 
@@ -269,10 +261,10 @@ var gComposePane = {
 
   initLanguageMenu: function ()
   {
+    var languageMenuList = document.getElementById("languageMenuList");
     this.mSpellChecker = Components.classes['@mozilla.org/spellchecker/myspell;1'].getService(Components.interfaces.mozISpellCheckingEngine);
     var o1 = {};
     var o2 = {};
-    var languageMenuList = document.getElementById('LanguageMenulist');
 
     // Get the list of dictionaries from
     // the spellchecker.
@@ -282,74 +274,117 @@ var gComposePane = {
     var dictList = o1.value;
     var count    = o2.value;
 
+    // if we don't have any dictionaries installed, disable the menu list
+    languageMenuList.disabled = !count;
+
+    // If dictionary count hasn't changed then no need to update the menu.
+    if (this.mDictCount == count)
+      return;
+
+    // Store current dictionary count.
+    this.mDictCount = count;
+
     // Load the string bundles that will help us map
     // RFC 1766 strings to UI strings.
 
     // Load the language string bundle.
     var languageBundle = document.getElementById("languageBundle");
-    var regionBundle;
+    var regionBundle = null;
     // If we have a language string bundle, load the region string bundle.
     if (languageBundle)
       regionBundle = document.getElementById("regionBundle");
-  
+
     var menuStr2;
     var isoStrArray;
-    var defaultItem = null;
     var langId;
+    var langLabel;
     var i;
 
-    for (i = 0; i < dictList.length; i++)
+    for (i = 0; i < count; i++)
     {
       try {
         langId = dictList[i];
         isoStrArray = dictList[i].split("-");
 
-        dictList[i] = new Array(2); // first subarray element - pretty name
-        dictList[i][1] = langId;    // second subarray element - language ID
-
         if (languageBundle && isoStrArray[0])
-          dictList[i][0] = languageBundle.getString(isoStrArray[0].toLowerCase());
+          langLabel = languageBundle.getString(isoStrArray[0].toLowerCase());
 
-        if (regionBundle && dictList[i][0] && isoStrArray.length > 1 && isoStrArray[1])
+        if (regionBundle && langLabel && isoStrArray.length > 1 && isoStrArray[1])
         {
           menuStr2 = regionBundle.getString(isoStrArray[1].toLowerCase());
           if (menuStr2)
-            dictList[i][0] = dictList[i][0] + "/" + menuStr2;
+            langLabel += "/" + menuStr2;
         }
 
-        if (dictList[i][0] && isoStrArray.length > 2 && isoStrArray[2])
-          dictList[i][0] += " (" + isoStrArray[2] + ")";
+        if (langLabel && isoStrArray.length > 2 && isoStrArray[2])
+          langLabel += " (" + isoStrArray[2] + ")";
 
-        if (!dictList[i][0])
-          dictList[i][0] = dictList[i][1];
+        if (!langLabel)
+          langLabel = langId;
       } catch (ex) {
-        // GetString throws an exception when
-        // a key is not found in the bundle. In that
-        // case, just use the original dictList string.
+        // getString throws an exception when a key is not found in the
+        // bundle. In that case, just use the original dictList string.
+        langLabel = langId;
+      }
+      dictList[i] = [langLabel, langId];
+    }
 
-        dictList[i][0] = dictList[i][1];
+    // sort by locale-aware collation
+    dictList.sort(
+      function compareFn(a, b)
+      {
+        return a[0].localeCompare(b[0]);
+      }
+    );
+
+    // Remove any languages from the list.
+    var languageMenuPopup = languageMenuList.firstChild;
+    while (languageMenuPopup.hasChildNodes())
+      languageMenuPopup.removeChild(languageMenuPopup.firstChild);
+
+    // append the dictionaries to the menu list...
+    for (i = 0; i < count; i++)
+      languageMenuList.appendItem(dictList[i][0], dictList[i][1]);
+
+    languageMenuList.setInitialSelection();
+  },
+
+  populateFonts: function()
+  {
+    var fontsList = document.getElementById("FontSelect");
+    try
+    {
+      var enumerator = Components.classes["@mozilla.org/gfx/fontenumerator;1"]
+                                 .getService(Components.interfaces.nsIFontEnumerator);
+      var localFontCount = { value: 0 }
+      var localFonts = enumerator.EnumerateAllFonts(localFontCount);
+      for (var i = 0; i < localFonts.length; ++i)
+      {
+        if (localFonts[i] != "")
+          fontsList.appendItem(localFonts[i], localFonts[i]);
       }
     }
-  
-    // note this is not locale-aware collation, just simple ASCII-based sorting
-    // we really need to add loacel-aware JS collation, see bug XXXXX
-    dictList.sort();
+    catch(e) { }
+   },
 
-    var curLang  = languageMenuList.value;
+   restoreHTMLDefaults: function()
+   {
+     // reset throws an exception if the pref value is already the default so
+     // work around that with some try/catch exception handling
+     try {
+       document.getElementById('msgcompose.font_face').reset();
+     } catch (ex) {}
 
-    // now select the dictionary we are currently using
-    for (i = 0; i < dictList.length; i++)
-    {
-      var item = languageMenuList.appendItem(dictList[i][0], dictList[i][1]);
-      if (curLang && dictList[i][1] == curLang)
-        defaultItem = item;
-    }
+     try {
+       document.getElementById('msgcompose.font_size').reset();
+     } catch (ex) {}
 
-    // Now make sure the correct item in the menu list is selected.
-    if (defaultItem)
-      languageMenuList.selectedItem = defaultItem;
-    else
-      languageMenuList.selectedIndex = 0;
-      
-  },
+     try {
+       document.getElementById('msgcompose.text_color').reset();
+     } catch (ex) {}
+
+     try {
+       document.getElementById('msgcompose.background_color').reset();
+     } catch (ex) {}
+   },  
 };
