@@ -1,4 +1,4 @@
-//@line 41 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/base/content/searchBar.js"
+//@line 41 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/base/content/searchBar.js"
 
 var gSearchSession = null;
 var gPreQuickSearchView = null;
@@ -10,7 +10,6 @@ var gSearchInProgress = false;
 var gSearchInput = null;
 var gDefaultSearchViewTerms = null;
 var gQSViewIsDirty = false;
-var gHighlightedMessageText = false; 
 var gIgnoreFocus = false;
 var gIgnoreClick = false;
 var gNumTotalMessages;
@@ -25,16 +24,9 @@ const kQuickSearchSubject = 0;
 const kQuickSearchSender = 1;
 const kQuickSearchSenderOrSubject = 2;
 const kQuickSearchBody = 3;
-const kQuickSearchHighlight = 4;
+// const kQuickSearchHighlight = 4; // * We no longer support this quick search mode..*
 const kQuickSearchRecipient = 5;
 
-var gFinder = Components.classes["@mozilla.org/embedcomp/rangefind;1"].createInstance()
-                        .QueryInterface(Components.interfaces.nsIFind);
-// Colors for highlighting
-var gHighlightColors = new Array("yellow", "lightpink", "aquamarine", 
-                                 "darkgoldenrod", "darkseagreen", "lightgreen", 
-                                 "rosybrown", "seagreen", "chocolate", "violet");
-
 
 function SetQSStatusText(aNumHits)
 {
@@ -71,11 +63,10 @@ var gSearchNotificationListener =
         statusFeedback.showProgress(0);
         gStatusBar.setAttribute("mode","normal");
         gSearchInProgress = false;
-        viewDebug("gSearchInput = " + gSearchInput.value + "\n");
 
         // ### TODO need to find out if there's quick search within a virtual folder.
         if (gCurrentVirtualFolderUri &&
-         (gSearchInput.value == "" || gSearchInput.showingSearchCriteria))
+         (!gSearchInput || gSearchInput.value == "" || gSearchInput.showingSearchCriteria))
         {
           var vFolder = GetMsgFolderFromUri(gCurrentVirtualFolderUri, false);
           var dbFolderInfo = vFolder.getMsgDatabase(msgWindow).dBFolderInfo;
@@ -84,11 +75,6 @@ var gSearchNotificationListener =
           vFolder.updateSummaryTotals(true); // force update from db.
           var msgdb = vFolder.getMsgDatabase(msgWindow);
           msgdb.Commit(MSG_DB_LARGE_COMMIT);
-
-          // load the last used mail view for the folder...
-          var result = dbFolderInfo.getUint32Property("current-view", 0);
-          ViewChangeByValue(result);
-
           // now that we have finished loading a virtual folder, scroll to the correct message
           ScrollToMessageAfterFolderLoad(vFolder);
         }
@@ -150,7 +136,12 @@ function createQuickSearchView()
       var saveViewSearchListener = gDBView.QueryInterface(Components.interfaces.nsIMsgSearchNotify);
       gSearchSession.unregisterListener(saveViewSearchListener);
     }
-    CreateDBView(gDBView.msgFolder, (gXFVirtualFolderTerms) ? nsMsgViewType.eShowVirtualFolderResults : nsMsgViewType.eShowQuickSearchResults, gDBView.viewFlags, gDBView.sortType, gDBView.sortOrder);
+    // if grouped by sort, turn that off, as well as threaded, since we don't
+    // group quick search results yet.
+    var viewFlags = gDBView.viewFlags;
+    if (viewFlags & nsMsgViewFlagsType.kGroupBySort)
+      viewFlags &= ~(nsMsgViewFlagsType.kGroupBySort | nsMsgViewFlagsType.kThreadedDisplay);
+    CreateDBView(gDBView.msgFolder, (gXFVirtualFolderTerms) ? nsMsgViewType.eShowVirtualFolderResults : nsMsgViewType.eShowQuickSearchResults, viewFlags, gDBView.sortType, gDBView.sortOrder);
   }
 }
 
@@ -178,12 +169,8 @@ function initializeSearchBar()
 
 function onEnterInSearchBar()
 {
-  viewDebug ("onEnterInSearchBar gSearchInput.value = " + gSearchInput.value + " showing criteria = " + gSearchInput.showingSearchCriteria +"\n");
-   if (gSearchInput.value == "" || gSearchInput.showingSearchCriteria) 
-   {
-     if (gSearchInput.searchMode == kQuickSearchHighlight)
-       removeHighlighting();
-     
+   if (!gSearchInput || gSearchInput.value == "" || gSearchInput.showingSearchCriteria) 
+   { 
      if (gDBView.viewType == nsMsgViewType.eShowQuickSearchResults 
         || gDBView.viewType == nsMsgViewType.eShowVirtualFolderResults)
      {
@@ -204,24 +191,20 @@ function onEnterInSearchBar()
      else if (gPreQuickSearchView && !gDefaultSearchViewTerms)// may be a quick search from a cross-folder virtual folder
       restorePreSearchView();
      
+     if (gSearchInput)
      gSearchInput.showingSearchCriteria = true;
      
      gQSViewIsDirty = false;
      return;
    }
 
-   if (gSearchInput.searchMode == kQuickSearchHighlight)
-     highlightMessage(true);
-   else
-   {
-     initializeSearchBar();
+   initializeSearchBar();
 
-     ClearThreadPaneSelection();
-     ClearMessagePane();
+   ClearThreadPaneSelection();
+   ClearMessagePane();
 
-     onSearch(null);
-     gQSViewIsDirty = false;
-  }
+   onSearch(null);
+   gQSViewIsDirty = false;
 }
 
 function restorePreSearchView()
@@ -265,7 +248,9 @@ function restorePreSearchView()
     gPreQuickSearchView = null;    
   }
   else //create default view type
+  {
     CreateDBView(folder, nsMsgViewType.eShowAllThreads, viewFlags, sortType, sortOrder);
+  }
 
   RerootThreadPane();
    
@@ -323,6 +308,7 @@ function createSearchTermsWithList(aTermsArray)
   var searchTermsArray = searchTerms.QueryInterface(Components.interfaces.nsISupportsArray);
   searchTermsArray.Clear();
 
+  var i;
   var selectedFolder = GetThreadPaneFolder();
   var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                   .getService(Components.interfaces.nsIIOService);
@@ -338,7 +324,7 @@ function createSearchTermsWithList(aTermsArray)
       var srchFolderUri = dbFolderInfo.getCharPtrProperty("searchFolderUri");
       viewDebug("createSearchTermsWithList xf vf scope = " + srchFolderUri + "\n");
       var srchFolderUriArray = srchFolderUri.split('|');
-      for (var i in srchFolderUriArray) 
+      for (i in srchFolderUriArray) 
       {
         var realFolderRes = GetResourceFromUri(srchFolderUriArray[i]);
         var realFolder = realFolderRes.QueryInterface(Components.interfaces.nsIMsgFolder);
@@ -354,7 +340,7 @@ function createSearchTermsWithList(aTermsArray)
   }
 
   // add each item in termsArray to the search session
-  for (var i = 0; i < termsArray.Count(); i++)
+  for (i = 0; i < termsArray.Count(); ++i)
     gSearchSession.appendTerm(termsArray.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgSearchTerm));
 }
 
@@ -363,7 +349,7 @@ function getScopeToUse(aTermsArray, aFolderToSearch, aIsOffline)
   if (aIsOffline || aFolderToSearch.server.type != 'imap')
     return nsMsgSearchScope.offlineMail;
 
-  var scopeToUse = gSearchInput.searchMode == kQuickSearchBody && !gSearchInput.showingSearchCriteria
+  var scopeToUse = gSearchInput && gSearchInput.searchMode == kQuickSearchBody && !gSearchInput.showingSearchCriteria
                    ? nsMsgSearchScope.onlineMail : nsMsgSearchScope.offlineMail;
 
   // it's possible one of our search terms may require us to use an online mail scope (such as imap body searches)
@@ -505,6 +491,7 @@ function onSearchKeyPress(event)
 
 function onSearchInputFocus(event)
 {
+  GetSearchInput();
   // search bar has focus, ...clear the showing search criteria flag
   if (gSearchInput.showingSearchCriteria)
   {
@@ -512,39 +499,60 @@ function onSearchInputFocus(event)
     gSearchInput.showingSearchCriteria = false;
   }
   
-  if (gIgnoreFocus)
+  if (gIgnoreFocus) // got focus via mouse click, don't need to anything else
     gIgnoreFocus = false;
   else
+  {
     gSearchInput.select();
+    gQuickSearchFocusEl = gSearchInput;   // only important that this be non-null
+  }
 }
 
 function onSearchInputMousedown(event)
 {
+  GetSearchInput();
   if (gSearchInput.hasAttribute("focused")) 
+  {
     gIgnoreClick = true;
+
+    // already focused, don't need to restore focus elsewhere (if the Clear button was clicked)
+    // ##HACK## Need to check 'clearButtonHidden' because the field is blurred when it has
+    // focus and the hidden button is clicked, in which case we want to perform the normal
+    // onBlur function.
+    gQuickSearchFocusEl = gSearchInput.clearButtonHidden ? gSearchInput : null;
+  }
   else 
   {
     gIgnoreFocus = true;
     gIgnoreClick = false;
-    gSearchInput.setSelectionRange(0, 0);
+
+    // save the last focused element so that focus can be restored (if the Clear button was clicked)
+    gQuickSearchFocusEl = gLastFocusedElement;
   }
+  // (if Clear button was clicked, onClearSearch() is called before onSearchInputClick())
 }
 
 
 function onSearchInputClick(event)
 {
-  if (!gIgnoreClick && gSearchInput.selectionStart == gSearchInput.selectionEnd)
-    gSearchInput.select();
+  if (!gIgnoreClick)
+  {
+    gQuickSearchFocusEl = null; // ##HACK## avoid onSearchInputBlur() side effects
+    gSearchInput.select();      // ## triggers onSearchInputBlur(), but focus returns to field
+  }
+  if (!gQuickSearchFocusEl)
+    gQuickSearchFocusEl = gSearchInput;   // mousedown wasn't on Clear button
 }
 
 function onSearchInputBlur(event)
 { 
-  if (gQuickSearchFocusEl && gQuickSearchFocusEl.id == 'searchInput') // ignore the blur if we are in the middle of processing the clear button
+  if (!gQuickSearchFocusEl) // ignore the blur if we are in the middle of processing the clear button
     return;
 
+  gQuickSearchFocusEl = null;
   if (!gSearchInput.value)
     gSearchInput.showingSearchCriteria = true;
-    
+
   if (gSearchInput.showingSearchCriteria)
     gSearchInput.setSearchCriteriaText();
 }
@@ -561,7 +569,6 @@ function onSearchInput(returnKeyHit)
 
   // only select the text when the return key was hit
   if (returnKeyHit) {
-    GetSearchInput();
     gSearchInput.select();
     onEnterInSearchBar();
   }
@@ -578,34 +585,26 @@ function onClearSearch()
 {
   if (!gSearchInput.showingSearchCriteria) // ignore the text box value if it's just showing the search criteria string
   {
-    gQuickSearchFocusEl = gLastFocusedElement;  //save of the last focused element so that focus can be restored
-  Search("");
+    Search("");
+    gIgnoreClick = true;
     // this needs to be on a timer otherwise we end up messing up the focus while the Search("") is still happening
-    setTimeout("restoreSearchFocusAfterClear();", 0); 
+    if (gQuickSearchFocusEl) // set in onSearchInputMouseDown
+      setTimeout("restoreSearchFocusAfterClear();", 0); 
   }
 }
 
 function restoreSearchFocusAfterClear()
 {
-  gQuickSearchFocusEl.focus();
-  gSearchInput.clearButtonHidden = 'true';
-  gQuickSearchFocusEl = null;
+  if (gQuickSearchFocusEl)
+    gQuickSearchFocusEl.focus();
 }
 
+// called from commandglue.js in cases where the view is being changed and QS
+// needs to be cleared.
 function ClearQSIfNecessary()
 {
-  GetSearchInput();
-
-  if (gSearchInput.value == "")
+  if (!gSearchInput || gSearchInput.showingSearchCriteria)
     return;
-
-  Search("");
-}
-
-// called after the user switches folders while inside
-// of a quick search view...
-function clearQuickSearchAfterFolderChange()
-{
   gSearchInput.setSearchCriteriaText();
 }
 
@@ -615,8 +614,6 @@ function Search(str)
   if (gSearchInput.showingSearchCriteria && str != "")
     return;
 
-  GetSearchInput();
-
   if (str != gSearchInput.value)
   {
     gQSViewIsDirty = true; 
@@ -627,36 +624,7 @@ function Search(str)
   onSearchInput(true);
 }
 
-// When the front end has finished loading a virtual folder, it calls openVirtualFolder
-// to actually perform the folder search. We use this method instead of calling Search("") directly
-// from FolderLoaded in order to avoid moving focus into the search bar.
-function loadVirtualFolder()
-{
-  // bit of a hack...if the underlying real folder is loaded with the same view value
-  // as the value for the virtual folder being searched, then ViewChangeByValue
-  // fails to change the view because it thinks the view is already correctly loaded.
-  // so set gCurrentViewValue back to All. 
-  gCurrentViewValue = 0; 
-  onEnterInSearchBar();
-}
-
-// this notification gets generated from layout when it finishes laying out a message
-// in the message pane. 
-function onQuickSearchNewMsgLoaded()
-{
-  // if we are in highlighting mode and we have highlight text in the search box then 
-  // re-highlight this new message.
-  // Optimization: We'll special case Message Body quick searches and highlight those as well as find in message
-  // searches.
-  if ( (gSearchInput.searchMode == kQuickSearchHighlight || gSearchInput.searchMode == kQuickSearchBody)
-       && gSearchInput.value && !gSearchInput.showingSearchCriteria)
-  {
-    highlightMessage(false);
-  }
-}
-
 // helper methods for the quick search drop down menu
-
 function changeQuickSearchMode(aMenuItem)
 {
   viewDebug("changing quick search mode\n");
@@ -677,11 +645,7 @@ function changeQuickSearchMode(aMenuItem)
   else if (gSearchInput.showingSearchCriteria) // if we are showing criteria text and the box isn't empty, change the criteria text
     gSearchInput.setSearchCriteriaText();     
   else if (oldSearchMode != gSearchInput.searchMode) // the search mode just changed so we need to redo the quick search
-  {
-    if (gHighlightedMessageText)
-      removeHighlighting(); // remove any existing highlighting in the message before switching gears
     onEnterInSearchBar();
-  }
 }
 
 function saveViewAsVirtualFolder()
@@ -694,116 +658,10 @@ function InitQuickSearchPopup()
   // disable the create virtual folder menu item if the current radio
   // value is set to Find in message since you can't really  create a VF from find
   // in message
-
-  if (gSearchInput.searchMode == 4 /* find in page */ || gSearchInput.value == "" || gSearchInput.showingSearchCriteria)
+  
+  GetSearchInput();  
+  if (!gSearchInput ||gSearchInput.value == "" || gSearchInput.showingSearchCriteria)
     document.getElementById('quickSearchSaveAsVirtualFolder').setAttribute('disabled', 'true');
   else
     document.getElementById('quickSearchSaveAsVirtualFolder').removeAttribute('disabled');
 }
-
-// Methods to support highlighting. Most of this was shamelessly copied from the mozdev google toolbar project
-function highlightMessage(removeExistingHighlighting)
-{
-  // remove any existing highlighting
-  if (removeExistingHighlighting)
-    removeHighlighting(); 
-
-  // wanted to use selection to extend to the word
-  // and compare with the found range, in order to match
-  // only whole words
-    
-  // Save selection
-  // XXX: Note to self, we may want to break on | and white space here
-  // even though normal quick searches treat white spaces as signficant
-  var termList = gSearchInput.value.split("|");
-  for (var i = 0; i < termList.length; i++) 
-      highlight(termList[i], gHighlightColors[i %10]);
-  
-  gHighlightedMessageText = true;
-}
-
-function removeHighlighting()
-{
-  if (!gHighlightedMessageText)
-    return;
-
-  var msgDocument = window.top.content;
-  var doc = msgDocument.document;
-  var elem = null;
-  while ((elem = doc.getElementById('mail-highlight-id'))) 
-  {
-    var child = null;
-    var docfrag = doc.createDocumentFragment();
-    var next = elem.nextSibling;
-    var parent = elem.parentNode;
-    while((child = elem.firstChild))
-      docfrag.appendChild(child);
-  
-    parent.removeChild(elem);
-    parent.insertBefore(docfrag, next);
-  }  
-
-  gHighlightedMessageText = false;
-
-  return;
-}
-
-function highlight(word, color)
-{
-  var msgDocument = window.top.content;
-
-  var doc = msgDocument.document;
-  if (!doc) 
-    return;
-  
-  if (!("body" in doc))
-    return;
-  
-  var body = doc.body; 
-  var count = body.childNodes.length;
-  searchRange = doc.createRange();
-  startPt = doc.createRange();
-  endPt = doc.createRange();
-
-  var baseNode = doc.createElement("span");
-  baseNode.setAttribute("style", "background-color: " + color + ";");
-  baseNode.setAttribute("id", "mail-highlight-id");
-
-  searchRange.setStart(body, 0);
-  searchRange.setEnd(body, count);
-
-  startPt.setStart(body, 0);
-  startPt.setEnd(body, 0);
-  endPt.setStart(body, count);
-  endPt.setEnd(body, count);
-  highlightText(word, baseNode);
-}
-
-// search through the message looking for occurrences of word
-// and highlighting them. 
-function highlightText(word, baseNode)
-{
-  var retRange = null;
-  while((retRange = gFinder.Find(word, searchRange, startPt, endPt))) 
-  {
-    // Highlight
-    var nodeSurround = baseNode.cloneNode(true);
-    var node = highlightRange(retRange, nodeSurround);
-    startPt = node.ownerDocument.createRange();
-    startPt.setStart(node, node.childNodes.length);
-    startPt.setEnd(node, node.childNodes.length);
-  }
-}
-
-function highlightRange(range, node)
-{
-  var startContainer = range.startContainer;
-  var startOffset = range.startOffset;
-  var endOffset = range.endOffset;
-  var docfrag = range.extractContents();
-  var before = startContainer.splitText(startOffset);
-  var parent = before.parentNode;
-  node.appendChild(docfrag);
-  parent.insertBefore(node, before);
-  return node;
-}
