@@ -1,4 +1,4 @@
-//@line 40 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/mail/base/content/widgetglue.js"
+//@line 40 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/mail/base/content/widgetglue.js"
 
 /*
  * widget-specific wrapper glue. There should be one function for every
@@ -129,9 +129,7 @@ function MsgCompactFolder(isAll)
       if (expungedBytes > 0)
       { 
         if (gDBView)
-        {
           gCurrentlyDisplayedMessage = gDBView.currentlyDisplayedMessage;
-        }
 
         ClearThreadPaneSelection();
         ClearThreadPane();
@@ -197,7 +195,10 @@ function onEditVirtualFolderPropertiesCallback(aVirtualFolderURI)
   }
 }
 
-function MsgFolderProperties() 
+/**
+ @param tabID  initial tab
+ */
+function MsgFolderProperties(tabID)
 {
   var preselectedURI = GetSelectedFolderURI();
   var msgFolder = GetMsgFolderFromUri(preselectedURI, true);
@@ -227,7 +228,25 @@ function MsgFolderProperties()
               {preselectedURI:preselectedURI, serverType:serverType,
               msgWindow:msgWindow, title:windowTitle,
               okCallback:FolderProperties, 
-              tabID:"", tabIndex:0, name:name});
+              tabID:tabID, name:name,
+              rebuildSummaryCallback:RebuildSummaryFile});
+}
+
+function RebuildSummaryFile(msgFolder)
+{
+  var msgDB = msgFolder.getMsgDatabase(msgWindow);
+  msgDB.summaryValid = false;
+  msgFolder.ForceDBClosed();
+  // these two lines will cause the thread pane to get reloaded
+  // when the download/reparse is finished. Only do this
+  // if the selected folder is loaded (i.e., not thru the
+  // context menu on a non-loaded folder).
+  if (msgFolder == GetLoadedMsgFolder())
+  {
+    gRerootOnFolderLoad = true;
+    gCurrentFolderToReroot = msgFolder.URI;
+  }
+  msgFolder.updateFolder(msgWindow);
 }
 
 function FolderProperties(name, oldName, uri)
@@ -250,7 +269,7 @@ function MsgToggleMessagePane()
 
 function MsgSetFolderCharset() 
 {
-  MsgFolderProperties() 
+  MsgFolderProperties();
 }
 
 // Given a URI we would like to return corresponding message folder here.
