@@ -1,4 +1,4 @@
-//@line 37 "/usr/obj/ports/mozilla-thunderbird-1.5.0.12/mozilla/toolkit/mozapps/extensions/content/about.js"
+//@line 37 "/usr/obj/ports/mozilla-thunderbird-2.0.0.6/mozilla/toolkit/mozapps/extensions/content/about.js"
 
 function EM_NS(aProperty)
 {
@@ -71,28 +71,39 @@ function init()
   
   var extensionHomepage = document.getElementById("extensionHomepage");
   if (homepage) {
-    extensionHomepage.setAttribute("href", homepage);
+    extensionHomepage.setAttribute("homepageURL", homepage);
     extensionHomepage.setAttribute("tooltiptext", homepage);
   }
   else
     extensionHomepage.hidden = true;
     
-  var contributorsBox = document.getElementById("contributorsBox");
-  var contributorsArc = rdfs.GetResource(EM_NS("contributor"));
-  var contributors = gExtensionDB.GetTargets(extension, contributorsArc, true);
-  var count = 0;
-  while (contributors.hasMoreElements()) {
-    var contributor = contributors.getNext().QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
-    var label = document.createElement("label");
-    label.setAttribute("value", contributor);
-    label.setAttribute("class", "contributor");
-    contributorsBox.appendChild(label);
-    ++count;
+  var sectionProps = [["developersBox", "developer", "extensionDevelopers"],
+                      ["translatorsBox", "translator", "extensionTranslators"],
+                      ["contributorsBox", "contributor", "extensionContributors"]];
+
+  for (var i = 0; i < sectionProps.length; ++i) {
+    var node = document.getElementById(sectionProps[i][0]);
+    var arc = rdfs.GetResource(EM_NS(sectionProps[i][1]));
+    var targets = gExtensionDB.GetTargets(extension, arc, true);
+    if (!(targets.hasMoreElements()))
+      document.getElementById(sectionProps[i][2]).hidden = true;
+    else {
+      while (targets.hasMoreElements()) {
+        var literalValue = targets.getNext().QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
+        var label = document.createElement("label");
+        label.setAttribute("value", literalValue);
+        label.setAttribute("class", "contributor");
+        node.appendChild(label);
+      }
+    }
   }
-  if (count == 0)
-    document.getElementById("extensionContributors").hidden = true;
     
   var acceptButton = document.documentElement.getButton("accept");
   acceptButton.label = extensionsStrings.getString("aboutWindowCloseButton");
 }
 
+function loadHomepage(aEvent) {
+  window.close();
+  window.opener.openURL(aEvent.target.getAttribute("homepageURL"));
+}
+
